/*******************************************************************************
 * Copyright (c) 2011, 2012 SINTEF, Martin F. Johansen.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     SINTEF, Martin F. Johansen - the implementation
 *******************************************************************************/

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import no.sintef.ict.splcatool.FileUtility;

import org.eclipse.emf.ecore.EObject;

import cvl.AND_;
import cvl.CVLModel;
import cvl.CompositeVariability;
import cvl.Constraint;
import cvl.ExecutablePrimitiveTerm;
import cvl.IMPLIES;
import cvl.Iterator;
import cvl.NOT;
import cvl.OR;
import cvl.VariabilitySpecification;

public class CVL2GDSL {
	static public void main(String [] args) throws FileNotFoundException, IOException{
		File cvlFile = new File(args[0]);
		String gdslstr = cvlToGDSL(cvlFile);
		new FileUtility().writeStringToFile(args[0] + ".m", gdslstr);
		System.out.println(gdslstr);
	}

	public static String cvlToGDSL(File cvlFile){
		CVLModel cvlmodel = CVLUtils.loadCVLModel(cvlFile);
		
		//System.out.println(cvlmodel);
		
		VariabilitySpecification vs = cvlmodel.getVariabilitySpecification();
		
		//System.out.println(vs.getName());
		
		String r = rec(vs, 0, null);
		
		List<String> cs = getConstraints(vs);
		if(cs.size() > 0){
			r += "\n%%\n\n";
			for(String c : cs){
				r += c + ";\n";
			}
		}
		
		//System.out.println("\n\n--------------------------");
		//System.out.println(r);
		
		return r;
	}
	
	private static String rec(VariabilitySpecification root, int level, VariabilitySpecification preroot){
		String ret = "";
		String prer = "";
		
		if(root instanceof Constraint)
			return ret;
		
		if(root instanceof Iterator){
			Iterator i = (Iterator) root;
			if(i.getLowerLimit() == 0 && i.getUpperLimit() == 1){
			}
			if(i.getLowerLimit() == 1 && i.getUpperLimit() == 1){
				prer = preroot.getName().replace(" ", "_") + "_";
			}
			if(i.getLowerLimit() == 1 && i.getUpperLimit() == -1){
				prer = preroot.getName().replace(" ", "_") + "_";
			}
		}
		
		ret += prer + root.getName().replace(" ", "_") + " : ";
		
		for(EObject x : root.eContents()){
			if(x instanceof VariabilitySpecification){
				if(x instanceof CompositeVariability){
					String sep = " ";
					String post = "";
					String pre = "";
					if(root instanceof Iterator){
						Iterator i = (Iterator) root;
						if(i.getLowerLimit() == 0 && i.getUpperLimit() == 1){
							sep = "|";
						}
						if(i.getLowerLimit() == 1 && i.getUpperLimit() == 1){
							sep = "|";
						}
						if(i.getLowerLimit() == 1 && i.getUpperLimit() == -1){
							sep = "|";
						}
					}
					if(x instanceof Iterator){
						Iterator i = (Iterator) x;
						if(i.getLowerLimit() == 0 && i.getUpperLimit() == 1){
							post = "*";
						}
						if(i.getLowerLimit() == 1 && i.getUpperLimit() == -1){
							post = "+";
							pre = root.getName().replace(" ", "_") + "_";
						}
						if(i.getLowerLimit() == 1 && i.getUpperLimit() == 1){
							pre = root.getName().replace(" ", "_") + "_";
						}
					}
					CompositeVariability cv = (CompositeVariability) x;
					ret += pre + cv.getName().replace(" ", "_") + post + sep;
				}
			}
		}
		
		if(ret.endsWith(": ")){
			ret = "";
		}else{
			String postmark = " :: _" + root.getName().replace(" ", "_");
			if(root instanceof Iterator){
				Iterator i = (Iterator) root;
				//System.out.println("Checking " + root);
				if(i.getLowerLimit() == 0 && i.getUpperLimit() == 1){
					//postmark = "";
				}
				if(i.getLowerLimit() == 1 && i.getUpperLimit() == 1){
					postmark = "";
				}
				if(i.getLowerLimit() == 1 && i.getUpperLimit() == -1){
					postmark = "";
				}
			}
			ret = ret.substring(0, ret.length()-1) + postmark + ";\n";
		}
		
		for(EObject x : root.eContents()){
			if(x instanceof VariabilitySpecification){
				/*for(int i = 0; i < level; i++)
					System.out.print("\t");
				*/
				if(x instanceof CompositeVariability){
					CompositeVariability cv = (CompositeVariability) x;
					//System.out.println(/*root.getName() + ": " + */cv.getName());
				}
				
				VariabilitySpecification vs = (VariabilitySpecification) x;
				//System.out.println(/*root.getName() + ": " + */vs.getName() + ", " + vs.getClass());
				ret += rec(vs, level + 1, root);
			}
		}
		
		return ret;
	}
	
	static public List<String> getConstraints(EObject root){
		List<String> cs = new ArrayList<String>();
		
		for(EObject z : root.eContents()){
			if(z instanceof Constraint){
				cs.add(getConstraintString((Constraint) z));
			}else
				cs.addAll(getConstraints(z));
		}
		return cs;
	}

	static private String getConstraintString(Constraint z) {
		String res = "";
		
		if(z instanceof IMPLIES){
			IMPLIES i = (IMPLIES) z;
			res = "(" + getConstraintString(i.getLeft()) + ") implies (" + getConstraintString(i.getRight()) + ")";
		}else if(z instanceof AND_){
			AND_ i = (AND_) z;
			res = "(" + getConstraintString(i.getLeft()) + ") and (" + getConstraintString(i.getRight()) + ")";
		}else if(z instanceof OR){
			OR i = (OR) z;
			res = "(" + getConstraintString(i.getLeft()) + ") or (" + getConstraintString(i.getRight()) + ")";
		}else if(z instanceof NOT){
			NOT n = (NOT) z;
			res = "not (" + getConstraintString(n.getTerm()) + ")";
		}else if(z instanceof cvl.ExecutablePrimitiveTerm){
			ExecutablePrimitiveTerm e = (ExecutablePrimitiveTerm) z;
			res = e.getPrimitive().getName().replace(" ", "_");
		}
		
		return res;
	}
}
