/*******************************************************************************
 * Copyright (c) 2011, 2012 SINTEF, Martin F. Johansen.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     SINTEF, Martin F. Johansen - the implementation
 *******************************************************************************/

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import no.sintef.ict.splcatool.CSVException;
import no.sintef.ict.splcatool.CoveringArray;
import no.sintef.ict.splcatool.CoveringArrayFile;

import org.eclipse.emf.ecore.EObject;

import cvl.CVLModel;
import cvl.CompositeVariability;
import cvl.Constraint;
import cvl.Iterator;
import cvl.VariabilitySpecification;
import cvl.runtime.CVLResolutionModelManager;


public class InjectCAIntoCVL {
	static public void main(String [] args) throws IOException, CSVException{
		injectCAIntoCVL(new File(args[0]), new CoveringArrayFile(args[2]), new File(args[1]));
	}

	public static void injectCAIntoCVL(File cvlFile, CoveringArrayFile ca, File outputFile){
		// Load model
		CVLModel cvlmodel = CVLUtils.loadCVLModel(cvlFile);
		Map<String, VariabilitySpecification> vss = new HashMap<String, VariabilitySpecification>();
		getAllVS(cvlmodel.getVariabilitySpecification(), vss);
		
		// Remove resolutions
		cvlmodel.getResolutionSpecification().clear();
		
		// Add products from covering array
		int i = 1;
		for(List<Integer> x : ca.getSolutionsAsList()){
			
			// Get product configuration
			//System.out.println(x);
			List<String> names = new ArrayList<String>();
			List<EObject> elements = new ArrayList<EObject>();
			for(int y : x){
				if(y > 0){
					names.add(ca.getId(y));
					if(vss.get(ca.getId(y)) != null)
						elements.add(vss.get(ca.getId(y)));
				}
			}
			//System.out.println(names);
			
			CVLResolutionModelManager resManager = new CVLResolutionModelManager();
			List<String> errorList = new ArrayList<String>();
			elements.add(cvlmodel.getVariabilitySpecification());
			cvlmodel = resManager.runResolutionBuilder(elements, ca.getName(i), errorList, null);
			i++;
		}
	
		//System.out.println(cvlmodel.getResolutionSpecification().size());
		
		//System.out.println(errorList);
		
		// Save model
		CVLUtils.saveCVLModel(cvlmodel, outputFile);
	}
	
	private static void getAllVS(VariabilitySpecification root, Map<String, VariabilitySpecification> all){
		for(EObject x : root.eContents()){
			if(x instanceof VariabilitySpecification){
				if(x instanceof Constraint) continue;
				VariabilitySpecification vs = (VariabilitySpecification) x;
				all.put(vs.getName().replace(" ", "_"), vs);
				getAllVS(vs, all);
			}
		}
	}
}
