package no.sintef.ict.splcatool;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeoutException;

import no.sintef.ict.splcatool.CoveringArrayFile.Type;

public abstract class CoveringArray {
	protected Map<Integer, String> nrid;
	protected Map<String, Integer> idnr;
	protected int t;
	protected List<Integer> btr = new ArrayList<Integer>();
	protected List<List<Integer>> initial = new ArrayList<List<Integer>>();
	protected List<String> featureOrder = new ArrayList<String>();
	
	protected boolean coverZerosOnly = true;
	protected boolean coverOnlyOnes = false;
	protected boolean firstHalfOnly = false;
	protected boolean secondHalfOnly = false;
	protected boolean firstFourthOnly = false;
	protected boolean thirdFourthOnly = false;
	protected int coverEight = 0;

	abstract public void generate() throws TimeoutException, CoveringArrayGenerationException;
	abstract public void generate(int i, Integer sizelimit) throws TimeoutException, CoveringArrayGenerationException;
	
	abstract public Integer[] getRow(int n);
	abstract public int getRowCount();
	abstract public void setTimeout(int timeout);
	
	@Override
	public boolean equals(Object cabo) {
		CoveringArray cab = (CoveringArray) cabo;
		CoveringArray caa = this;
		
		if(caa.getRowCount() != cab.getRowCount())
			return false;
		for(int i = 0; i<caa.getRowCount(); i++){
			Integer[]rowa = caa.getRow(i);
			Integer[]rowb = cab.getRow(i);
			if(!Arrays.equals(rowa, rowb)){
				return false;
			}
		}
		
		return true;
	}

	
	public void writeToFile(String filename) throws FileNotFoundException, IOException {
		writeToFile(filename, Type.horizontal);
	}
	
	public void writeToFile(String filename, Type type) throws FileNotFoundException, IOException {
		StringBuffer output = new StringBuffer();
		
		if(type == Type.vertical){
			List<Integer> nrs = new ArrayList<Integer>(nrid.keySet());
			Collections.sort(nrs);
			
			output.append("Product\\Feature;");
			
			for(int nr : nrs){
				output.append(nrid.get(nr) + ";");
			}
			output.append("\n");
			for(int i = 0; i < getRowCount(); i++){
				output.append(i + ";");
				for(int n : getRow(i)){
					if(n==0)
						output.append("X;");
					else
						output.append("-;");
				}
				if(isRowBTR(i)) output.append("BTR;");
				output.append("\n");
			}
			
			new FileUtility().writeStringToFile(filename, output.toString());
		}else if(type == Type.horizontal){
			List<Integer> nrs = new ArrayList<Integer>(nrid.keySet());
			Collections.sort(nrs);
			
			output.append("Feature\\Product;");
			for(int i = 0; i < getRowCount(); i++){
				output.append(i + ";");
			}
			output.append("\n");
			
			// Cache rows
			List<Integer[]> rows = new ArrayList<Integer[]>();
			for(int j = 0; j < getRowCount(); j++){
				rows.add(getRow(j));
			}
			
			for(int i = 0; i < nrs.size(); i++){
				output.append(nrid.get(nrs.get(i)) + ";");
				for(int j = 0; j < getRowCount(); j++){
					int n = rows.get(j)[i];
					if(n==0)
						output.append("X;");
					else
						output.append("-;");
				}
				output.append("\n");
			}
			
			// Sort if featureOrder is populated
			if(featureOrder.size() > 0){
				//System.out.println("Sorting");
				String lines[] = output.toString().split("\n");
				
				output = new StringBuffer(lines[0] + "\n");
				
				for(String id : featureOrder){
					//System.out.println(id);
					for(int i = 1; i < lines.length; i++){
						String line = lines[i];
						if(line.split(";")[0].equals(id)){
							output.append(line + "\n");
							break;
						}
					}
				}
			}
		}
		
		// Write to file
		new FileUtility().writeStringToFile(filename, output.toString());
	}
	
	abstract public double estimateGenerationTime();
	public int getT() {
		return t;
	}
	
	public boolean isRowBTR(int i){
		return btr.contains(i);
	}
	
	public int getRowCountBTR(){
		return getRowCount()-btr.size();
	}
	
	abstract public String getAlgorithmName();
	public void startFrom(CoveringArray partial) {
		for(int n = 0; n < partial.getRowCount(); n++){
			// Convert
			Integer[] solinteger = partial.getRow(n);
			List<Integer> sol = new ArrayList<Integer>();
			for(int i = 0; i < solinteger.length; i++){
				int x = idnr.get(partial.nrid.get(i+1));
				if(solinteger[i]==1) x = -x;
				sol.add(x);
			}
			initial.add(sol);
		}
		
		List<Integer> nrs = new ArrayList<Integer>(partial.idnr.values());
		Collections.sort(nrs);
		for(Integer n : nrs){
			featureOrder.add(partial.nrid.get(n));
		}
		//System.out.println(featureOrder);
	}
	abstract public long getCoverage();
	
	public List<List<Integer>> getSolutionsAsList(){
		List<List<Integer>> sols = new ArrayList<List<Integer>>();
		for(int i=0;i<getRowCount();i++){
			Integer rowa[] = getRow(i);
			List<Integer> row = new ArrayList<Integer>();
			for(int j=0;j<rowa.length;j++)
				row.add((rowa[j]==0)?(j+1):-(j+1));
			sols.add(row);
		}
		return sols;
	}
	
	public String getId(int i) {
		return nrid.get(i);
	}
	
	public void coverZeros(boolean doCover) {
		coverZerosOnly = doCover;
	}
	
	public void coverOnlyOnes(boolean coverOnes) {
		this.coverOnlyOnes  = coverOnes;
	}
	public void coverFirstHalfOnly(boolean b) {
		firstHalfOnly = b;
	}
	public void coverSecondHalfOnly(boolean b) {
		secondHalfOnly = b;
	}
	public void coverFirstFourthOnly(boolean b) {
		firstFourthOnly = b;
	}
	public void coverThirdFourthOnly(boolean b) {
		thirdFourthOnly = b;
	}
	public void coverEightOnly(int coverEight) {
		this.coverEight  = coverEight;
	}
}
