/*******************************************************************************
 * Copyright (c) 2011, 2012 SINTEF, Martin F. Johansen.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     SINTEF, Martin F. Johansen - the implementation
 *******************************************************************************/

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import no.sintef.ict.splcatool.CSVException;
import no.sintef.ict.splcatool.CoveringArrayFile;
import no.sintef.ict.splcatool.FileUtility;

import org.junit.Test;


public class TestLASPLTCVL {
	@Test
	public void testInjectActual() throws IOException, CSVException, InterruptedException{
		String cvlInputFileName = "TestData/Realistic/ABB/SafetyModule.noRes.cvl";
		String caFileName = "TestData/Realistic/ABB/SafetyModule.cvl.m.actual.csv";
		String cvlOutputFileName = "TestData/Realistic/ABB/SafetyModule.gen.cvl";
		String cvlOriginalFileName = "TestData/Realistic/ABB/SafetyModule.original.cvl";
		String dir = "TestData/Realistic/ABB/";
		
		InjectCAIntoCVL.injectCAIntoCVL(
			new File(cvlInputFileName), 
			new CoveringArrayFile(caFileName), 
			new File(cvlOutputFileName)
		);
		
		{
			String origFile = new FileUtility().readFileAsString(cvlOriginalFileName);
			String newFile = new FileUtility().readFileAsString(cvlOutputFileName);
			
			assertEquals(origFile, newFile);
		}
		
		// Generate product models
		{
			for(int i = 0; i < 3; i++){
				new File("TestData/Realistic/ABB/product"+i+".uml").delete();
			}
			
			Runtime rt = Runtime.getRuntime();
			rt.exec("\"C:/Program Files (x86)/Java/jdk1.6.0_23/bin/java\" -jar ../../../lib/eclipse-cvl-headless/plugins/org.eclipse.equinox.launcher_*.jar -data workspace -application org.variabilitymodeling.cvl.headless.HeadlessCVL SafetyModule.gen.cvl product.uml", null, new File(dir).getAbsoluteFile());
		}
		
		// Wait for output to be generated
		for(int i = 0; i < 3; i++){
			while(!new File("TestData/Realistic/ABB/product"+i+".uml").exists());
		}
		
		Thread.sleep(1000);
		
		// Check
		for(int i = 0; i < 3; i++){
			String origFile = new FileUtility().readFileAsString("TestData/Realistic/ABB/test"+i+".uml");
			String newFile = new FileUtility().readFileAsString("TestData/Realistic/ABB/product"+i+".uml");
			
			assertEquals(origFile, newFile);
		}
	}
	
	@Test(timeout=60000)
	public void testGenAndInject() throws IOException, CSVException, SecurityException, IllegalArgumentException, NoSuchMethodException, IllegalAccessException, InvocationTargetException, ClassNotFoundException, InterruptedException{
		String cvlInputFileName = "TestData/Realistic/ABB/SafetyModule.original.cvl";
		String gdslFileName = "TestData/Realistic/ABB/SafetyModule.cvl.m";
		String caOutputName = "TestData/Realistic/ABB/SafetyModule.cvl.m.ca2.csv";
		String cvlOutputFileName = "TestData/Realistic/ABB/SafetyModule.ca2.cvl";
		String dir = "TestData/Realistic/ABB/";
		
		CoveringArrayFile ca = new CoveringArrayFile(caOutputName);
		
		// Extract FM
		String gdslStr = CVL2GDSL.cvlToGDSL(new File(cvlInputFileName));
		new FileUtility().writeStringToFile(gdslFileName, gdslStr);
		
		/* Generate Covering Array */{
			Runtime rt = Runtime.getRuntime();
			Process p = rt.exec("java -jar lib/SPLCATool-v0.3-SPLC2012.jar -t t_wise -s 2 -a J11 -fm " + gdslFileName + " -o " + caOutputName);
			do{ // Wait for it to finish
				try{
					p.exitValue();
				}catch(IllegalThreadStateException e){
					continue;
				}
			}while(false);
		}
		
		// Inject covering array
		InjectCAIntoCVL.injectCAIntoCVL(
			new File(cvlInputFileName), 
			ca, 
			new File(cvlOutputFileName)
		);
		
		/* Generate product models */{
			for(int i = 0; i < ca.getRowCount(); i++){
				new File("TestData/Realistic/ABB/product"+i+".uml").delete();
			}
			
			Runtime rt = Runtime.getRuntime();
			rt.exec("\"C:/Program Files (x86)/Java/jdk1.6.0_23/bin/java\" -jar ../../../lib/eclipse-cvl-headless/plugins/org.eclipse.equinox.launcher_*.jar -data workspace -application org.variabilitymodeling.cvl.headless.HeadlessCVL SafetyModule.ca2.cvl product.uml", null, new File(dir).getAbsoluteFile());
		}
		
		// Wait for output to be generated
		for(int i = 0; i < ca.getRowCount(); i++){
			while(!new File("TestData/Realistic/ABB/product"+i+".uml").exists());
		}
	}
}
