/*******************************************************************************
 * Copyright (c) 2011, 2012 SINTEF, Martin F. Johansen.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     SINTEF, Martin F. Johansen - the implementation
 *******************************************************************************/

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import no.sintef.ict.splcatool.CSVException;
import no.sintef.ict.splcatool.CoveringArrayFile;
import no.sintef.ict.splcatool.FileUtility;


public class CVLLASPLT {
	static FileUtility fu = new FileUtility();
	
	public static void main(String[] args) throws IOException, CSVException, InterruptedException {
		
		// --------------------------------------------------------------------
		
		// Extract FM
		System.out.print("Extracting Feature Model from CVL Model: ");
		String gdslStr = CVL2GDSL.cvlToGDSL(new File(CVLLASPLTInput.cvlInputModelName));
		new FileUtility().writeStringToFile(CVLLASPLTInput.gdslFileName, gdslStr);
		System.out.println("Done");
		
		/* Generate Covering Array */
		System.out.print("Generating "+CVLLASPLTInput.t+"-wise Covering Array: ");
		{
			Runtime rt = Runtime.getRuntime();
			//Process p = rt.exec("java -jar "+CVLLASPLTInput.splCATool+" -t t_wise -s "+CVLLASPLTInput.t+" -a J11 -fm " + CVLLASPLTInput.gdslFileName + " -o " + CVLLASPLTInput.caOutputName, null, new File(CVLLASPLTInput.wDir));
			Process p = rt.exec("java -jar "+CVLLASPLTInput.splCATool+" -t t_wise -s "+CVLLASPLTInput.t+" -a J11 -fm " + CVLLASPLTInput.gdslFileName + " -o " + CVLLASPLTInput.caOutputName + " -startFrom " + CVLLASPLTInput.startProducts, null, new File(CVLLASPLTInput.wDir));
			
			InputStream inp = p.getInputStream();
			InputStream err = p.getErrorStream();

			StringBuffer stdout = new StringBuffer();
			StringBuffer stderr = new StringBuffer();
			
			while(true){
				while(inp.available() > 0){
					stdout.append((char)inp.read());
				}
				
				while(err.available() > 0){
					stderr.append((char)err.read());
				}
				
				try{
					System.out.println(p.exitValue());
					/*System.out.println(stdout);
					System.out.println(stderr);*/
				}catch(IllegalThreadStateException e){
					Thread.sleep(100);
					continue;
				}
				
				break;
			}
		}
		System.out.println("Done");
		
		// Inject covering array
		System.out.print("Injecting Covering Array into CVL model: ");
		CoveringArrayFile ca = new CoveringArrayFile(CVLLASPLTInput.caOutputName);
		InjectCAIntoCVL.injectCAIntoCVL(
			new File(CVLLASPLTInput.cvlInputModelName), 
			ca, 
			new File(CVLLASPLTInput.cvlOutputFileName)
		);
		System.out.println("Done");
		
		/* Generate product models */
		System.out.println("Executing CVL: ");
		{
			for(Integer i = 0; i < ca.getRowCount(); i++){
				new File(CVLLASPLTInput.product.replace("?", i.toString())).delete();
			}
			
			Runtime rt = Runtime.getRuntime();
			String command = 
				CVLLASPLTInput.java32+" "+
				"-jar "+CVLLASPLTInput.cvlHeadless+" "+
				"-data "+CVLLASPLTInput.wsTmp+" "+
				"-application org.variabilitymodeling.cvl.headless.HeadlessCVL "+
				CVLLASPLTInput.cvlOutputFileName + " " +
				CVLLASPLTInput.productModelName;
			rt.exec(command, null, new File(CVLLASPLTInput.wDir));
		}
		
		// Wait for output to be generated
		for(Integer i = 0; i < ca.getRowCount(); i++){
			while(!new File(CVLLASPLTInput.product.replace("?", i.toString())).exists());
			
			System.out.println("\tGenerated " + CVLLASPLTInput.product.replace("?", i.toString()));
		}
		System.out.println("Done");
		
		// For each product 
		for(Integer i = 0; i < ca.getRowCount(); i++){
		//for(Integer i = 2; i < 3; i++){
			String next = CVLLASPLTInput.product.replace("?", i.toString());
			System.out.println(next);
			
			System.out.println("\tMoving to " + CVLLASPLTInput.target);
			
			String contents = fu.readFileAsString(next);
			fu.writeStringToFile(CVLLASPLTInput.target, contents);
			
			// Delete all test in test folder
			final String testFolder = CVLLASPLTInput.ws + "/" + CVLLASPLTInput.targetProject + "/test/" + CVLLASPLTInput.testPackage;
			File testDir = new File(testFolder);
/*			for(String s : testDir.list()){
				new File(testFolder + "/" + s).delete();
				//System.out.println("deleted " + testFolder + "/" + s);
			}
*/
			// Delete java files
			for(File f : new File(CVLLASPLTInput.ws + "/" + CVLLASPLTInput.targetProject + "/generated/SafetyDrive/").listFiles()){
				f.delete();
			}
			
			// Run JavaFrame
			Runtime rt = Runtime.getRuntime();
			String command = 
				CVLLASPLTInput.java32+" -jar " +
				CVLLASPLTInput.jfHeadless + " " + 
				"-data "+CVLLASPLTInput.ws+" "+
				"-application no.uio.ifi.javaframetransformation.headless.HeadlessJavaFrame " +
				CVLLASPLTInput.target;
			
			//System.out.println(command);
			System.out.print("\tRunning JavaFrame Generator: Exit code: ");
			Process p = rt.exec(command);
			
			InputStream inp = p.getInputStream();
			InputStream err = p.getErrorStream();
			
			StringBuffer stdout = new StringBuffer();
			StringBuffer stderr = new StringBuffer();
			
			while(true){
				while(inp.available() > 0){
					stdout.append((char)inp.read());
				}
				
				while(err.available() > 0){
					stderr.append((char)err.read());
				}
				
				try{
					System.out.println(p.exitValue());
				}catch(IllegalThreadStateException e){
					Thread.sleep(100);
					continue;
				}
				
				break;
			}
			
			// Investigate results from JavaFrame
			if(stdout.toString().contains("Transformation failed")){
				String s = stdout.toString();
				System.out.println("\tJavaFrameError: " + s.substring(s.indexOf("Transformation failed")));
				continue;
			}
			
/*			if(stderr.toString().contains("Transformation failed")){
				String s = stderr.toString();
				System.out.println("stderr: " + s.substring(s.indexOf("Transformation failed")));
				continue;
			}
*/			
			// Copy relevant tests
			final String testSrcFolder = CVLLASPLTInput.wDir + "tests/";
/*			for(String s : new File(testSrcFolder).list()){
				String c = fu.readFileAsString(testSrcFolder + s);
				fu.writeStringToFile(testFolder + "/" + s, c);
				//System.out.println("Wrote to " + testFolder + "/" + s + " from " + testSrcFolder + s);
			}
*/
			// Delete tests
			for(File f : new File(CVLLASPLTInput.ws + "/" + CVLLASPLTInput.targetProject + "/tests-generated/SafetyDrive/").listFiles()){
				f.delete();
			}
			
			// Launch Anatoly's tool
			
			{
				//java -jar plugins\org.eclipse.equinox.launcher_*.jar -data <workspace_directory> -application no.sintef.plugin.utp.headless <project_name> <relative_path_to_product0.uml> <relative_path_to_product1.uml> <relative_path_to_productN.uml>
				
				rt = Runtime.getRuntime();
				command = 
					CVLLASPLTInput.java32+" -jar " +
					CVLLASPLTInput.atHeadless + " " + 
					"-data "+CVLLASPLTInput.ws+" "+
					"-application no.sintef.plugin.utp.headless " +
					CVLLASPLTInput.targetProject + " " + 
					"product.uml";
				
				//System.out.println(command);
				System.out.print("\tRunning Anatoly's Tool: Exit code: ");
				p = rt.exec(command);
				
				inp = p.getInputStream();
				err = p.getErrorStream();
				
				stdout = new StringBuffer();
				stderr = new StringBuffer();
				
				while(true){
					while(inp.available() > 0){
						stdout.append((char)inp.read());
					}
					
					while(err.available() > 0){
						stderr.append((char)err.read());
					}
					
					try{
						System.out.println(p.exitValue());
						/*System.out.println(stdout);
						System.out.println(stderr);*/
					}catch(IllegalThreadStateException e){
						Thread.sleep(100);
						continue;
					}
					
					break;
				}
			}
			
			//System.out.println(CVLLASPLTInput.ws + "/" + CVLLASPLTInput.targetProject + "/tests-generated/SafetyDrive/");
			
			List<String> tests = new ArrayList<String>();
			for(File f : new File(CVLLASPLTInput.ws + "/" + CVLLASPLTInput.targetProject + "/tests-generated/SafetyDrive/").listFiles()){
				String fm = f.getAbsolutePath();
				fm = fm.substring(fm.lastIndexOf("\\")+1, fm.lastIndexOf("."));
				//System.out.println("SafetyDrive."+fm);
				tests.add("SafetyDrive."+fm);
			}
			
			// Add tests according to configuration
/*			String tfms = fu.readFileAsString("TestFeatureMapping.csv");
			int count = 0;
			for(String tfm : tfms.split("\r\n")){
				count++;
				if(count == 1) continue;
				
				String es[] = tfm.split(";");
				//System.out.println("Feature " + es[0] + " is " + ca.isIncluded(i, es[0]));
				
				if(ca.isIncluded(i, es[0])){
					//System.out.println("\t\tCopy " + testSrcFolder + es[1] + "/" + es[2] + ".java");
					//System.out.println("\t\t\tTo " + testFolder + "/" + es[2] + ".java");
					String c = fu.readFileAsString(testSrcFolder + es[1] + "/" + es[2] + ".java");
					fu.writeStringToFile(testFolder + "/" + es[2] + ".java", c);
				}
			}
*/			
			// Delete java files
			
			for(File f : new File(CVLLASPLTInput.ws + "/" + CVLLASPLTInput.targetProject + "/bin/SafetyDrive/").listFiles()){
				f.delete();
			}
			
			for(int r = 0 ; r < 2; r++){ // TODO: Must be run two times to ensure that workspace is refreshed to include added tests
			
			// Build WS
			System.out.print("\tBuilding workspace: ");
			command = 
				CVLLASPLTInput.eclipsec32 + " " +
				"-noSplash "+
				"-data "+CVLLASPLTInput.ws+" "+
				"-application org.eclipse.jdt.apt.core.aptBuild";
			//System.out.println(command);
			p = rt.exec(command);
			
			inp = p.getInputStream();
			err = p.getErrorStream();
			
			while(true){
				//System.out.println("waiting ... " + inp.available() + ", " + p.getErrorStream().available());
				
				while(inp.available() > 0){
					//System.out.print((char)inp.read());
					inp.read();
				}
				
				while(err.available() > 0){
					err.read();
				}
				
				try{
					System.out.println("Exit code: " + p.exitValue());
				}catch(IllegalThreadStateException e){
					Thread.sleep(1000);
					continue;
				}
				
				break;
			}
			
			}
			
			// Run tests
			System.out.println("\tRun Tests: ");
			
			// Run tests
			for(String s : tests){
				String testSuite = s;
				System.out.print("\t\t" + testSuite + ": ");

				command = 
					CVLLASPLTInput.java32 + " " +
					"-cp \""+CVLLASPLTInput.testClassPath+"\" " +
					"org.junit.runner.JUnitCore " + 
					testSuite;
				//System.out.println(command);
				
				p = rt.exec(command, null, new File(CVLLASPLTInput.ws + "/"+CVLLASPLTInput.targetProject+"/"));
				inp = p.getInputStream();
				err = p.getErrorStream();
				
				stdout = new StringBuffer();
				stderr = new StringBuffer();
				
				while(true){
					//System.out.println("waiting ... " + inp.available() + ", " + err.available());
					
					while(inp.available() > 0){
						//System.out.print((char)inp.read());
						//inp.read();
						stdout.append((char)inp.read());
					}
					
					while(err.available() > 0){
						//System.err.print((char)err.read());
						//err.read();
						stderr.append((char)err.read());
					}
					
					try{
						System.out.println((p.exitValue()==1) ? "[fail]" : "[success]");
						if(p.exitValue()==1){
							System.out.print(stdout);
						}
					}catch(IllegalThreadStateException e){
						Thread.sleep(100);
						continue;
					}
					
					break;
				}
			}
			
			//break;
		}
	}

}
