/*******************************************************************************
 * Copyright (c) 2011, 2012 SINTEF, Martin F. Johansen.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     SINTEF, Martin F. Johansen - the implementation
 *******************************************************************************/

package no.sintef.ict.splcatool.lasplt;
public class DisableSystemExitTest {
	/**
	 * Disables System.exit calls
	 */
	public void disableSystemExitCall() {
		final SecurityManager securityManager = new SecurityManager() {
			public void checkPermission(java.security.Permission permission) {
				// Forbid only exit status 1
				// If you want to forbid any kind of halt, just use "exitVM"
				if (permission.getName().contains("exitVM")) {
					throw new SecurityException(
							"System.exit calls not allowed!");
				}
			}
		};
		System.setSecurityManager(securityManager);
	}

	/**
	 * Enables System.exit calls
	 */
	public static void enableSystemExitCall() {
		System.setSecurityManager(null);
	}

	public static void main(String args[]) {
		DisableSystemExitTest t = new DisableSystemExitTest();
		t.disableSystemExitCall();
		try {
			// You should get an exception here...
			System.out.println("System.exit(1) invoked:");
			System.exit(1);
		} catch (Exception e) {
			System.out.println("System.exit(1) exception: " + e.getMessage());
		}

		try {
			// You should not get an exception here since only exit status 1 is
			// forbidden
			System.out.println("System.exit(2) invoked:");
			System.exit(2);
		} catch (Exception e) {
			System.out.println("System.exit(2) exception: " + e.getMessage());
		}
	}

}
