/*******************************************************************************
 * Copyright (c) 2011, 2012 SINTEF, Martin F. Johansen.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     SINTEF, Martin F. Johansen - the implementation
 *******************************************************************************/

package no.sintef.ict.splcatool.lasplt;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

public class ExecuteCommand implements Runnable {
	private static boolean stop;
	private ExecutionResult ret;
	private long timeout;
	private String command;
	private StringBuffer sb;
	private String dir;
	
	public ExecuteCommand(long timeout, String command, String dir, StringBuffer sb){
		this.timeout = timeout;
		this.command = command;
		this.sb = sb;
		this.dir = dir;
		
		stop = false;
	}
	
	public void stop(){
		stop = true;
	}

	@Override
	public void run() {
		try {
			ret = executeCommand(timeout, command, dir, sb);
		} catch (IOException e) {
			//e.printStackTrace();
		} catch (InterruptedException e) {
			//e.printStackTrace();
		}
	}
	
	ExecutionResult getResult(){
		return ret;
	}

	ExecutionResult executeCommand(long timeout, String command, String dir, StringBuffer sb) throws IOException, InterruptedException{
		Runtime rt = Runtime.getRuntime();
		Process p = rt.exec(command, null, new File(dir));
		InputStream inp = p.getInputStream();
		InputStream err = p.getErrorStream();
		
		ret = new ExecutionResult();
		ret.retvalue = 0;
		ret.timeout = false;
		ret.time = 0;
		
		while(true){
			//System.out.println("waiting ... " + inp.available() + ", " + err.available());
			
			if((long)ret.time > (long)timeout*1000){
				ret.retvalue = 0;
				ret.timeout = true;
				p.destroy();
				Thread.sleep(2000);
			}
			
			while(inp.available() > 0){
				//System.out.print((char)inp.read());
				sb.append((char) inp.read());
			}
			
			while(err.available() > 0){
				//System.err.print((char)err.read());
				sb.append((char) err.read());
				
			}
			
			if(stop){
				p.destroy();
				Thread.sleep(2000);
				break;
			}
			
			try{
				ret.retvalue = p.exitValue();
			}catch(IllegalThreadStateException e){
				Thread.sleep(100);
				ret.time += 100;
				continue;
			}

			break;
		}
		
		return ret;
	}
}
