/*******************************************************************************
 * Copyright (c) 2011, 2012 SINTEF, Martin F. Johansen.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     SINTEF, Martin F. Johansen - the implementation
 *******************************************************************************/

package no.sintef.ict.splcatool.lasplt;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import no.sintef.ict.splcatool.CoveringArrayFile;

class TestExecution{
	Integer product = null;
	String testClass = null;
	String feature = null;
	Integer tests = null;
	Integer successes = null;
	Integer failures = null;
	Integer errors = null;
	String errorlog = null;
	String duration = null;
	String startime = null;
}

public class GenerateWebView {
	static List<TestExecution> tes = new ArrayList<TestExecution>();
	
	static void addTestInformation(int product, String testClass, String feature, int tests, int successes, int failures, int errors, String errorlog, String time, String timestamp){
		TestExecution te = null;
		// Find test execution
		for(TestExecution tea : tes){
			if(tea.product == product && tea.testClass.equals(testClass)){
				te = tea;
				break;
			}
		}
		if(te == null){
			te = new TestExecution();
			te.product = product;
			te.testClass = testClass;
			tes.add(te);
			return;
		}
		
		// Add new information
		if(feature != null) te.feature = feature;
		if(tests != -1) te.tests = tests;
		if(successes != -1) te.successes = successes;
		if(failures != -1) te.failures = failures;
		if(errors != -1) te.errors = errors;
		if(errorlog != null) te.errorlog = errorlog;
		if(time != null) te.duration = time;
		if(timestamp != null) te.startime = timestamp;
	}
	
	public static void main(String [] args) throws FileNotFoundException, IOException{
		// Input
		String baseDir = args[0];
		String testpackages = baseDir+"/results/test-packages.csv";
		
		// Start
		String view = "<html><head><link rel='stylesheet' href='style.css'></head><body>";
		view += "<h1>View</h1>";
		
		// Show Feature Model	
		view += "<h2>Feature Model</h2>";
		view += "<p>Eclipse.m</p>";
		
		// Show Parameters
		String tps[] = new FileUtility().readFileAsString(baseDir+"/results/testparameters.txt").split(";");
		view += "<h2>Parameters</h2>";
		view += "<p>t: "+tps[0]+"</p>";
		view += "<p>timeout: "+tps[1]+"s</p>";
		
		// Show Configurations
		view += "<h2>Configurations</h2>";
		CoveringArrayFile ca = new CoveringArrayFile(baseDir+"/results/Eclipse.m.ca"+tps[0]+".csv"); // TODO: Parameterize this
		view += "<p>Total: "+ca.getRowCount()+"</p>";
		view += "<table><tr><td>Feature\\Product</td>";
		for(int i = 0; i < ca.getRowCount(); i++){
			view += "<td class='nr'>"+(i+1)+"</td>";	
		}
		view += "</tr>";
		for(int i = 0; i < ca.getSolutionsAsList().get(0).size(); i++){
			String fid = ca.getId(i+1);
			view += "<tr><td>"+fid+"</td>";
			for(int j = 0; j < ca.getRowCount(); j++){
				List<String> configuration = LTSPLT.getFeatures(ca, j);
				if(configuration.contains(fid)){
					view += "<td class='sign'>X</td>";
				}else{
					view += "<td class='sign'>-</td>";
				}
			}
			view += "</tr>";
		}
		view += "</table>";
		
		// Show Results
		view += "<h2>Test Suite Plan</h2>";
		view += "<table><tr><td>Test Suite\\Product</td>";
		for(int i = 0; i < ca.getRowCount(); i++){
			view += "<td class='nr'>"+(i+1)+"</td>";	
		}
		view += "</tr>";
		
		CSV testpackagescsv = new CSV(testpackages);
		
		for(int i = 0; i < ca.getSolutionsAsList().get(0).size(); i++){
			String fid = ca.getId(i+1);
			view += "<tr><td>"+fid+"</td>";
			for(int j = 0; j < ca.getRowCount(); j++){
				List<String> configuration = LTSPLT.getFeatures(ca, j);
				if(configuration.contains(fid)){
					List<TestData> tds = LTSPLT.getTestsDataForFeature(testpackagescsv, fid);
					view += "<td class='nr'>"+tds.size()+"</td>";
				}else{
					view += "<td class='sign'>-</td>";
				}
			}
			view += "</tr>";
		}
		
		view += "</table>";
		
		// Show Results
		view += "<h2>Test Results for each Feature</h2>";
		view += "<table><tr><td>Test Suite\\Product</td>";
		for(int i = 0; i < ca.getRowCount(); i++){
			view += "<td class='nr'>"+(i+1)+"</td>";	
		}
		view += "</tr>";
		
		int totaltests = 0;
		int totalerrors = 0;
		int totalfailures = 0;
		
		for(int i = 0; i < ca.getSolutionsAsList().get(0).size(); i++){
			String fid = ca.getId(i+1);
			view += "<tr><td>"+fid+"</td>";
			for(int j = 0; j < ca.getRowCount(); j++){
				List<String> configuration = LTSPLT.getFeatures(ca, j);
				if(configuration.contains(fid)){
					List<TestData> tds = LTSPLT.getTestsDataForFeature(testpackagescsv, fid);
					
					int success = 0;
					int errors = 0;
					int failures = 0;
					int tests = 0;
					for(TestData td : tds){
						String testResultFile = baseDir+"/results/product"+j+"-"+td.testClass+"-test-result.xml";
						if(new File(testResultFile).exists() && new File(testResultFile).length()>0){
							TestResults tr = LTSPLT.getTestResults(testResultFile);
							success += tr.success;
							errors += tr.errors;
							failures += tr.failures;
							tests += tr.tests;
							
							totaltests+=tr.tests;
							totalerrors+=tr.errors;
							totalfailures+=tr.failures;
						}
					}
					
					if(tests>0){
						int color = 128 + success*127/tests;
						String colorbit = Integer.toHexString(color);
						String colorstr = "#" + colorbit + colorbit + colorbit;
						view += "<td class='nr' bgcolor='"+colorstr+"'>"+(errors+failures)+"</td>";
					}else if(tds.size()>0){
						view += "<td class='nr'>?</td>";
					}else{
						view += "<td class='nr'>-</td>";
					}
				}else{
					view += "<td> </td>";
				}
			}
			view += "</tr>";
		}
		
		view += "</table>";
		
		view += "<p>Tests: " + totaltests + "</p>";
		view += "<p>Errors: " + totalerrors + "</p>";
		view += "<p>Failures: " + totalfailures + "</p>";
		
		// Show Results
		view += "<h2>Test Results for each Test Suite</h2>";
		view += "<table><tr><td>Test Suite\\Product</td>";
		for(int i = 0; i < ca.getRowCount(); i++){
			view += "<td class='nr'>"+(i+1)+"</td>";	
		}
		view += "</tr>";
		
		// Gather test data
		Set<String> testClasses = new HashSet<String>();
		Map<String, String> tcfm = new HashMap<String, String>();
		for(int i = 0; i < ca.getSolutionsAsList().get(0).size(); i++){
			String fid = ca.getId(i+1);
			for(int j = 0; j < ca.getRowCount(); j++){
				List<String> configuration = LTSPLT.getFeatures(ca, j);
				if(configuration.contains(fid)){
					List<TestData> tds = LTSPLT.getTestsDataForFeature(testpackagescsv, fid);
					for(TestData td : tds){
						if(td.testClass.length()>0){
							testClasses.add(td.testClass);
							tcfm.put(td.testClass, fid);
						}
					}
				}
			}
		}
		
		for(String testClass : testClasses){
			view += "<tr><td>"+testClass+"</td>";
			for(int j = 0; j < ca.getRowCount(); j++){
				String testResultFile = baseDir+"/results/product"+j+"-"+testClass+"-test-result.xml";
				
				addTestInformation(j, testClass, tcfm.get(testClass), -1, -1, -1, -1, null, null, null);
				
				// check for error-log
				File dir = new File(baseDir+"/results");
				File errorlog = null;
				for(File f : dir.listFiles()){
					if(f.getName().contains("failure-"+j+"-"+testClass+".log")){
						//System.out.println(f.getName());
						errorlog = f;
					}
				}
				
				String errormsg = ""; 
				if(errorlog != null && errorlog.length() > 0){
					errormsg += "<a class='superscript' href='"+errorlog.getAbsolutePath()+"'>e</a>";
					addTestInformation(j, testClass, tcfm.get(testClass), -1, -1, -1, -1, errorlog.getAbsolutePath(), null, null);
					
					String wsrmsg = "The workspace exited with unsaved changes in the previous session; refreshing workspace to recover changes.";
					String elog = new FileUtility().readFileAsString(errorlog.getAbsolutePath());
					if(elog.contains(wsrmsg)){
						errormsg += "<a class='superscript' href='"+errorlog.getAbsolutePath()+"'>wsref</a>";
					}
				}
				
				if(new File(testResultFile).exists() && new File(testResultFile).length()==0){
					view += "<td class='nr' bgcolor='lightyellow'>&gt;"+tps[1]+"s</td>";
					addTestInformation(j, testClass, tcfm.get(testClass), -1, -1, -1, -1, null, null, null);
				}else if(new File(testResultFile).exists() && new File(testResultFile).length()>0){
					TestResults tr = LTSPLT.getTestResults(testResultFile);
					int color = 128 + tr.success*127/tr.tests;
					String colorbit = Integer.toHexString(color);
					String colorstr = "#" + colorbit + colorbit + colorbit;
					view += "<td class='nr' bgcolor='"+colorstr+"'>"+tr.errors+", "+tr.failures+errormsg+"</td>";
					
					addTestInformation(j, testClass, tcfm.get(testClass), tr.tests, tr.success, tr.failures, tr.errors, null, tr.time, tr.timestamp);
				}else{
					view += "<td class='nr'></td>";
					addTestInformation(j, testClass, tcfm.get(testClass), -1, -1, -1, -1, null, null, null);
				}
				
			}
			
			view += "</tr>";
		}
		
		view += "</table>";
		
		// Show times
		double totaltime = 0;
		view += "<h2>Test Times for each Test Suite</h2>";
		view += "<table><tr><td>Test Suite\\Product</td>";
		for(int i = 0; i < ca.getRowCount(); i++){
			view += "<td class='nr'>"+(i+1)+"</td>";	
		}
		view += "</tr>";
		
		for(String testClass : testClasses){
			view += "<tr><td>"+testClass+"</td>";
			for(int j = 0; j < ca.getRowCount(); j++){
				
				String testResultFile = baseDir+"/results/product"+j+"-"+testClass+"-test-result.xml";
				
				// check for error-log
				File dir = new File(baseDir+"/results");
				File errorlog = null;
				for(File f : dir.listFiles()){
					if(f.getName().contains("failure-"+j+"-"+testClass+".log")){
						//System.out.println(f.getName());
						errorlog = f;
					}
				}
				
				String errormsg = ""; 
				if(errorlog != null){
					errormsg += "<a class='superscript' href='"+errorlog.getAbsolutePath()+"'>e</a>";
				}
				
				if(new File(testResultFile).exists() && new File(testResultFile).length()==0){
					view += "<td class='nr' bgcolor='lightyellow'>&gt;"+tps[1]+"s</td>";
					totaltime += new Double(tps[1]);
				}else if(new File(testResultFile).exists() && new File(testResultFile).length()>0){
					TestResults tr = LTSPLT.getTestResults(testResultFile);
					String timestring = "";
					
					if(new Double(tr.time) < 1){
						timestring = "&lt;1s";
						totaltime += new Double(tr.time);
					}else{
						timestring = Math.round(new Double(tr.time))+"s";
						totaltime += new Double(tr.time);
					}
					
					view += "<td class='nr'>"+timestring+errormsg+"</td>";
				}else{
					view += "<td class='nr'></td>";
				}
				
			}
			
			view += "</tr>";
		}
		
		view += "</table>";
		
		view += "<h2>Total Test Time</h2>";
		view += "<p>"+Math.round(totaltime)+" s (" + (Math.round(totaltime)/3600) + " h)</p>";
		
		// Show test executon summaries
		view += "<h2>Test Class Summaries</h2>";
		view += "<table><tr><td>class</td><td>feature</td><td>tests</td><td>time</td><td>time per test</td><td>nonsuccesses</td><td>canbesuccess</td></tr>";
		for(String testClass : testClasses){
			view += "<tr><td>" + testClass + "</td>";
			
			// Feature
			String feature = null;
			for(TestExecution te : tes){
				if(!te.testClass.equals(testClass)) continue;
				feature = te.feature;
			}
			view += "<td>" + feature + "</td>";
			
			// Tests
			int tests = 0;
			for(TestExecution te : tes){
				if(! te.testClass.equals(testClass)) continue;
				if(te.tests == null) continue;
				if(te.tests > tests){
					tests = te.tests;
				}
			}
			view += "<td class='nr'>" + tests + "</td>";
			
			// time
			double time = 0;
			for(TestExecution te : tes){
				if(!te.testClass.equals(testClass)) continue;
				if(te.duration == null) continue;
				if(new Double(te.duration) > time){
					time = new Double(te.duration);
				}
			}
			view += "<td class='nr'>" + Math.round(time) + "s</td>";
			view += "<td class='nr'>" + Math.round(time/tests) + "s</td>";
			
			// nonsuccesses
			int nonsuccesses = 0;
			for(TestExecution te : tes){
				if(!te.testClass.equals(testClass)) continue;
				if(te.failures == null) continue;
				if(te.errors == null) continue;
				nonsuccesses += te.failures;
				nonsuccesses += te.errors;
			}
			String color = "";
			if(nonsuccesses == 0){
				color = " bgcolor='#DDFFDD' ";
			}
			view += "<td class='nr' "+color+">" + nonsuccesses + "</td>";
			
			// nonsuccesses
			boolean successfound = false;
			for(TestExecution te : tes){
				if(!te.testClass.equals(testClass)) continue;
				if(te.failures == null) continue;
				if(te.errors == null) continue;
				if(te.failures == 0 && te.errors == 0){
					successfound = true;
					if(testClass.equals("org.eclipse.jdt.ui.tests.LeakTestSuite")){
						System.out.println(te.product + ", " + te.duration);
					}
				}
			}
			if(successfound){
				color = " bgcolor='#DDFFDD' ";
			}
			view += "<td class='nr' "+color+">" + successfound + "</td>";
			
			// end
			view += "</tr>";
		}
		view += "</table>";
		
		// Write results
		new FileUtility().writeStringToFile(baseDir+"/webview/index.html", view);
	}
}
