/*******************************************************************************
 * Copyright (c) 2011, 2012 SINTEF, Martin F. Johansen.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     SINTEF, Martin F. Johansen - the implementation
 *******************************************************************************/

package no.sintef.ict.splcatool.lasplt;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.jar.Attributes;
import java.util.jar.JarFile;
import java.util.jar.Manifest;

public class RunJar {

	private String jar;
	private String[] args;
	private String className;

	public RunJar(String jar, String[] args) {
		this.jar = jar;
		this.args = args;
	}

	public void run() throws SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, InvocationTargetException, ClassNotFoundException, IOException {
		/*
		// Disable exit calls
		new DisableSystemExitTest().disableSystemExitCall();
		
		// Run Jar
		File file = new File(jar); // forward slashes with java.io.File works  
		URL[] urls = { file.toURI().toURL() };  
		URLClassLoader loader = new URLClassLoader(urls);
		
		JarFile jarFile = new JarFile(file);  
		Manifest manifest = jarFile.getManifest(); // warning: can be null  
		Attributes attributes = manifest.getMainAttributes();  
		String className = attributes.getValue(Attributes.Name.MAIN_CLASS);
		
		Class<?> cls = loader.loadClass(className); // replace the complete class name with the actual main class  
		Method main = cls.getDeclaredMethod("main", String[].class); // get the main method using reflection
		try{
			main.invoke(null, new Object[] {args}); // static methods are invoked with null as first argument
		}catch(InvocationTargetException e){
			
		}
		
		// Enable exit calls
		DisableSystemExitTest.enableSystemExitCall();
		*/
		
		//System.out.println("Running...");
		
		String oneArgs = "";
		for(String x : args)
			oneArgs += x + " ";
		
		System.out.println("java -jar -Xms40m -Xmx2g -XX:MaxPermSize=256m " + jar + " " + oneArgs);
		
	    try {
	    	String line;
	    	Process p = Runtime.getRuntime().exec("java -jar -Xms40m -Xmx2g -XX:MaxPermSize=256m " + jar + " " + oneArgs);
	    	InputStream is = p.getInputStream();
	    	InputStreamReader isr = new InputStreamReader(is);
	    	BufferedReader input = new BufferedReader(isr);
	    	//System.out.println("a");
	    	InputStream es = p.getErrorStream();
	    	InputStreamReader esr = new InputStreamReader(es);
	    	BufferedReader error = new BufferedReader(new InputStreamReader(p.getErrorStream()));
	    	//System.out.println("b");
	    	int waitseconds = 0;
	    	while(true){
	    		//System.out.println(is.available() + " and " + es.available());
	    		//int c = ' ';
	    		if(is.available() > 0){
	    			input.skip(is.available());
	    			//while ((c = isr.read()) != -1) {
	    	    		//System.out.print((char)c);
	    	    	//}
	    		}
	    		if(es.available() > 0){
	    			error.skip(es.available());
	    			//while ((c = esr.read()) != -1) {
	    	    		//System.out.print((char)c);
	    	    	//}
	    		}
	    		Thread.currentThread().sleep(1000);
	    		try{
	    			System.out.println("Exit value: " + p.exitValue());
	    			break;
	    		}catch(IllegalThreadStateException e){
	    			
	    		}
	    		waitseconds++;
	    		if(waitseconds==3600){
	    			System.out.println("Process timeout after 3600 seconds.");
	    			p.destroy();
	    		}
	    	}
	    	//System.out.println("wait over");
/*	    	int c = ' ';
	    	while ((c = isr.read()) != -1) {
	    		//System.out.print((char)c);
	    	}
/*	    	while ((c = esr.read()) != -1) {
	    		System.err.print((char)c);
	    	}
*/
/*	    	while ((line = input.readLine()) != null) {
	    		System.out.println("stdout:" + line);
	    	}
	    	System.out.println("c");
	    	while ((line = error.readLine()) != null) {
	    		System.out.println("stderr:" + line);
	    	}
	    	*/
	    	//System.out.println("d");
	    	
	    	input.close();
	    	isr.close();
	    	is.close();
	    	error.close();
	    	esr.close();
	    	es.close();
	    	//error.close();
	    	//System.out.println("e");
	    	
	    }catch (Exception err) {
	    	err.printStackTrace();
	    }
	      
	   // System.out.println("Done");
	}
}
