/*******************************************************************************
 * Copyright (c) 2011, 2012 SINTEF, Martin F. Johansen.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     SINTEF, Martin F. Johansen - the implementation
 *******************************************************************************/

package no.sintef.ict.splcatool.lasplt;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeoutException;

import splar.core.fm.FeatureModelException;

import de.ovgu.featureide.fm.core.io.UnsupportedModelException;

import no.sintef.ict.splcatool.CNF;
import no.sintef.ict.splcatool.CoveringArray;
import no.sintef.ict.splcatool.CoveringArrayFile;
import no.sintef.ict.splcatool.FileUtility;
import no.sintef.ict.splcatool.GUIDSL;
import no.sintef.ict.splcatool.SXFM;

public class SPLEclipseBased extends SoftwareProductLine {
	private GUIDSL fm;
	private SXFM sxfm;
	private CNF cnf;
	private String featureModel;
	private CSV packagescsv;
	private CSV testpackagescsv;
	private String base;
	private String launcher;
	private String zip;
	private String sevenZip;
	private String testZip;
	private String testZipSite;

	public SPLEclipseBased(String featureModel, CSV packagescsv, CSV testpackagescsv, String base, String launcher, String zip, String sevenZip, String testZip, String testZipSite) throws UnsupportedModelException, IOException, FeatureModelException {
		// Load Feature Model
		fm = new GUIDSL(featureModel);
		sxfm = fm.getSXFM();
		cnf = sxfm.getCNF();
		
		this.featureModel = featureModel;
		this.packagescsv = packagescsv;
		this.testpackagescsv = testpackagescsv;
		this.base = base;
		this.launcher = launcher;
		this.zip = zip;
		this.sevenZip = sevenZip;
		this.testZip = testZip;
		this.testZipSite = testZipSite;
	}

	@Override
	public void generateCoveringProducts(int t) throws TimeoutException, IOException, SecurityException, IllegalArgumentException, NoSuchMethodException, IllegalAccessException, InvocationTargetException, ClassNotFoundException {
		// Generate Covering Array
/*		CoveringArray ca = cnf.getCoveringArrayGenerator("J11", t, 1);
		ca.coverZeros(false);
		ca.generate();
		ca.writeToFile("ca"+t+".csv");
*/
		// Load Covering Array
		CoveringArray ca = new CoveringArrayFile("ca2.csv");
		
		// ---
		
		List<List<Integer>> sols = ca.getSolutionsAsList();
		System.out.println("Products: " + sols.size());
		
		StringBuffer times = new StringBuffer();
		
		// Build products
		for(int i = 0; i < sols.size(); i++){
			List<String> configuration = new ArrayList<String>();
			for(int j = 0; j < sols.get(i).size(); j++){
				if(sols.get(i).get(j) > 0)
					configuration.add(ca.getId(sols.get(i).get(j)));
			}
			
			long startTime = System.currentTimeMillis();
			Product p = new EclipseProduct(""+i, zip, base + "workspace/" + "Product" + i, sevenZip, base, launcher, packagescsv, testpackagescsv, configuration, testZip, testZipSite);
			long endTime = System.currentTimeMillis();
			
/*			times.append("Product" + i + ": " + (endTime-startTime) + " ms\n");
			new FileUtility().writeStringToFile("buildTime.txt", times.toString());
*/			
			addProduct(p);
		}
				
	}

}
