package cvl.constraints;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Stack;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.emf.common.util.EList;

import cvl.AND_;
import cvl.Assignment;
import cvl.CVLElement;
import cvl.CVLModel;
import cvl.CVLNamedElement;
import cvl.CompositeVariability;
import cvl.Constraint;
import cvl.ExecutablePrimitive;
import cvl.ExecutablePrimitiveTerm;
import cvl.FragmentSubstitution;
import cvl.IMPLIES;
import cvl.Invocation;
import cvl.Iterator;
import cvl.IteratorResolution;
import cvl.NOT;
import cvl.OR;
import cvl.ReferenceSubstitution;
import cvl.ResolutionElement;
import cvl.ResolutionValue;
import cvl.Type;
import cvl.ValueSubstitution;
import cvl.VariabilitySpecification;
import cvl.visitors.impl.TopDownVisitorImpl;

public class AlloyGenerator extends TopDownVisitorImpl<AlloyGeneratorContext, String> {
	
	protected Hashtable<String, CVLElement> objlookup = new Hashtable<String, CVLElement>();
	protected Hashtable<CVLElement, String> idlookup = new Hashtable<CVLElement, String>();
	protected int id_index = 0;
	Pattern indentifier = Pattern.compile("[a-zA-Z][a-zA-Z0-9[_]]*");
	
	protected Stack<String> parent = new Stack<String>();
	
	protected ArrayList<Invocation> invocations = new ArrayList<Invocation>();
	protected ArrayList<ResolutionElement> invocationResolutions = new ArrayList<ResolutionElement>();
	
	public String getNodeID(CVLNamedElement node) {
		String result = idlookup.get(node);
		if (result != null) return result;
		if (node.getName() != null && node.getName().length() > 0) {
            Matcher matcher = indentifier.matcher(node.getName().replace(' ', '_'));
            if(matcher.find()) result = matcher.group();
		}
		if (result != null) {
			if (objlookup.containsKey(result) && objlookup.get(result) != node) {
				result = result + "_AlloyUID" + id_index++;
			}
		}
		else result = node.eClass().getName() + "_AlloyUID" + id_index++;
		objlookup.put(result, node);
		idlookup.put(node, result);
		return result;
	}
	
	public String getAlloyPreambule() {
		String result = "/****************************************************************/\n\n";
		return result;
	}
	
	public String generateConstraints(CompositeVariability node, AlloyGeneratorContext context) {
		String result = "";
		for(Constraint c : selectConstraints(node.getVariabilitySpecification())) {
			result += "fact { " + c.accept(this, context) + " }\n";
		}
		return result;
	}
	
	@Override
	public String visitType(Type node, AlloyGeneratorContext context) {
		String result = getAlloyPreambule();
		String temp2 = "";
		result += "abstract sig Type_"+getNodeID(node)+"_Elements {}\n";
		result += "one sig Type_"+getNodeID(node)+" {\n";
		result += "elements : set Type_"+getNodeID(node)+"_Elements\n";
		result += "}\n";
		result += "fact { Type_"+getNodeID(node)+"_Elements in Type_"+getNodeID(node)+".elements }\n";
		
		ArrayList<ExecutablePrimitive> children = selectExecutablePrimitive(node.getBody());
		for(int i = 0 ; i<children.size(); i++) {
			ExecutablePrimitive e = children.get(i);
			temp2 += "fact{ all c: Type_"+getNodeID(node)+" | one e : "+getNodeID(e)+" | e in c.elements }\n";
		} 
		result += temp2 + "\n";
		parent.push("Type_"+getNodeID(node)+"_Elements");
		result += node.getBody().accept(this, context);
		parent.pop();
		return result;
	}
	

	public ArrayList<ExecutablePrimitive> selectExecutablePrimitive(EList<VariabilitySpecification> list) {
		ArrayList<ExecutablePrimitive> result = new ArrayList<ExecutablePrimitive>();
		for (VariabilitySpecification e : list) {
			if (e instanceof ExecutablePrimitive) result.add((ExecutablePrimitive)e);
		}
		return result;
	}
	
	public ArrayList<Constraint> selectConstraints(EList<VariabilitySpecification> list) {
		ArrayList<Constraint> result = new ArrayList<Constraint>();
		for (VariabilitySpecification e : list) {
			if (e instanceof Constraint) result.add((Constraint)e);
		}
		return result;
	}
	
	public ArrayList<ExecutablePrimitive> selectExecutablePrimitive(VariabilitySpecification spec) {
		ArrayList<ExecutablePrimitive> result = new ArrayList<ExecutablePrimitive>();
		if (spec instanceof ExecutablePrimitive) result.add((ExecutablePrimitive)spec);
		return result;
	}
	
	public String visitExecutablePrimitive(ExecutablePrimitive node, AlloyGeneratorContext context) {
		return "sig " + getNodeID(node) + " extends "+ parent.peek() +" {}\n";
	}
	
	@Override
	public String visitAND_(AND_ node, AlloyGeneratorContext context) {
		return "(" + node.getLeft().accept(this, context) + ") and (" + node.getRight().accept(this, context) + ")";
	}

	@Override
	public String visitAssignment(Assignment node, AlloyGeneratorContext context) {
		return visitExecutablePrimitive(node, context);
	}

	@Override
	public String visitCompositeVariability(CompositeVariability node, AlloyGeneratorContext context) {
		
		String result = "";
		String temp2 = "";
		result += "abstract sig "+getNodeID(node)+"_Elements {}\n";
		result += "sig "+getNodeID(node)+" extends " + parent.peek() + " {\n";
		result += "elements : set "+getNodeID(node)+"_Elements\n";
		result += "}\n";
		result += "fact { "+getNodeID(node)+"_Elements in "+getNodeID(node)+".elements }\n";
		
		ArrayList<ExecutablePrimitive> children = selectExecutablePrimitive(node.getVariabilitySpecification());
		for(int i = 0 ; i<children.size(); i++) {
			ExecutablePrimitive e = children.get(i);
			temp2 += "fact{ all c: "+getNodeID(node)+" | one e : "+getNodeID(e)+" | e in c.elements }\n";
		} 
		if (children.isEmpty())
			result += "fact { no "+getNodeID(node)+"_Elements }";
		result += temp2 + "\n";
		result += generateConstraints(node, context);
		parent.push(getNodeID(node)+"_Elements");
		for(VariabilitySpecification e : children)
			result += e.accept(this, context);
		parent.pop();
		return result;
	}

	@Override
	public String visitCVLModel(CVLModel node, AlloyGeneratorContext context) {
		String result = getAlloyPreambule();
		String temp2 = "";
		result += "abstract sig CVLModel_Elements {}\n";
		result += "one sig CVLModel {\n";
		result += "elements : set CVLModel_Elements\n";
		result += "}\n";
		result += "fact { CVLModel_Elements in CVLModel.elements }\n";
		
		ArrayList<ExecutablePrimitive> children = selectExecutablePrimitive(node.getVariabilitySpecification());
		for(int i = 0 ; i<children.size(); i++) {
			ExecutablePrimitive e = children.get(i);
			temp2 += "fact{ all c: CVLModel | one e : "+getNodeID(e)+" | e in c.elements }\n";
		} 
		result += temp2 + "\n";
		parent.push("CVLModel_Elements");
		if (node.getVariabilitySpecification() instanceof ExecutablePrimitive)
			result += node.getVariabilitySpecification().accept(this, context);
		parent.pop();
		return result;
	}

	@Override
	public String visitExecutablePrimitiveTerm(ExecutablePrimitiveTerm node, AlloyGeneratorContext context) {
		return "one " + getNodeID(node.getPrimitive());
	}

	@Override
	public String visitFragmentSubstitution(FragmentSubstitution node, AlloyGeneratorContext context) {
		return visitExecutablePrimitive(node, context);
	}

	@Override
	public String visitIMPLIES(IMPLIES node, AlloyGeneratorContext context) {
		return "(" + node.getLeft().accept(this, context) + ") implies (" + node.getRight().accept(this, context) + ")";
	}

	@Override
	public String visitInvocation(Invocation node, AlloyGeneratorContext context) {
		invocations.add(node);
		return visitExecutablePrimitive(node, context);
	}

	@Override
	public String visitIterator(Iterator node, AlloyGeneratorContext context) {
		String result = "";
		String temp2 = "";
		result += "abstract sig "+getNodeID(node)+"_Elements {}\n";
		result += "sig "+getNodeID(node)+" extends " + parent.peek() + " {\n";
		result += "elements : set "+getNodeID(node)+"_Elements\n";
		result += "}\n";
		result += "{\n";
		result += "  #elements >= " + node.getLowerLimit() + "\n";
		if (node.getUpperLimit()>=0)
			result += "  #elements <=" + node.getUpperLimit() + "\n";
		else{
			result += "  #elements <=" + node.getVariabilitySpecification().size() + "\n";
		}
		result += "}\n";
		result += "fact { "+getNodeID(node)+"_Elements in "+getNodeID(node)+".elements }\n";
		
		ArrayList<ExecutablePrimitive> children = selectExecutablePrimitive(node.getVariabilitySpecification());
		for(int i = 0 ; i<children.size(); i++) {
			ExecutablePrimitive e = children.get(i);
			temp2 += "fact{ all c: "+getNodeID(node)+" | lone e : "+getNodeID(e)+" | e in c.elements }\n";
		} 
		if (children.isEmpty())
			result += "fact { no "+getNodeID(node)+"_Elements }";
		if (node.isIsUnique())
			result += temp2 + "\n";
		result += generateConstraints(node, context);
		parent.push(getNodeID(node)+"_Elements");
		for(VariabilitySpecification e : children)
			result += e.accept(this, context);
		parent.pop();
		return result;
	}

	@Override
	public String visitNOT(NOT node, AlloyGeneratorContext context) {
		return "not ( " + node.getTerm().accept(this, context) + " )";
	}

	@Override
	public String visitOR(OR node, AlloyGeneratorContext context) {
		return "(" + node.getLeft().accept(this, context) + ") or (" + node.getRight().accept(this, context) + ")";
	}

	@Override
	public String visitReferenceSubstitution(ReferenceSubstitution node, AlloyGeneratorContext context) {
		return visitExecutablePrimitive(node, context);
	}

	@Override
	public String visitValueSubstitution(ValueSubstitution node, AlloyGeneratorContext context) {
		return visitExecutablePrimitive(node, context);
	}

	@Override
	public String visitIteratorResolution(IteratorResolution node, AlloyGeneratorContext context) {
		String result = "fact { one "+getNodeID(node.getIterator())+" }\n";
		for (ResolutionElement e : node.getChoice()) {
			result += e.accept(this, context);
		}
		return result;
	}
	
	@Override
	public String visitResolutionElement(ResolutionElement node, AlloyGeneratorContext context) {
		// We stop on invocations
		if (node.getElement() instanceof Invocation) {
			invocationResolutions.add(node);
			return "";		
		}
		String result = "fact { one "+getNodeID(node.getElement())+" }\n";
		for (IteratorResolution e : node.getResolution()) {
			result += e.accept(this, context);
		}
		return result;
	}

	@Override
	public String visitResolutionValue(ResolutionValue node, AlloyGeneratorContext context) {
		return "fact { one "+getNodeID(node.getElement())+" }\n";
	}
}
