package cvl.constraints;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.lang.model.element.ExecutableElement;

import org.eclipse.emf.common.util.EList;

import cvl.AND_;
import cvl.Assignment;
import cvl.CVLElement;
import cvl.CVLModel;
import cvl.CVLNamedElement;
import cvl.CompositeVariability;
import cvl.ExecutablePrimitive;
import cvl.ExecutablePrimitiveTerm;
import cvl.FragmentSubstitution;
import cvl.IMPLIES;
import cvl.Invocation;
import cvl.Iterator;
import cvl.IteratorResolution;
import cvl.NOT;
import cvl.OR;
import cvl.ReferenceSubstitution;
import cvl.ResolutionElement;
import cvl.ResolutionValue;
import cvl.Type;
import cvl.ValueSubstitution;
import cvl.VariabilitySpecification;
import cvl.visitors.impl.TopDownVisitorImpl;

public class AlloyGeneratorV1 extends TopDownVisitorImpl<AlloyGeneratorContext, String> {

	protected Hashtable<String, CVLElement> objlookup = new Hashtable<String, CVLElement>();
	protected Hashtable<CVLElement, String> idlookup = new Hashtable<CVLElement, String>();
	protected int id_index = 0;
	Pattern indentifier = Pattern.compile("[a-zA-Z][a-zA-Z0-9[_]]*");
	
	public String getNodeID(CVLNamedElement node) {
		String result = idlookup.get(node);
		if (result != null) return result;
		if (node.getName() != null && node.getName().length() > 0) {
            Matcher matcher = indentifier.matcher(node.getName().replace(' ', '_'));
            if(matcher.find()) result = matcher.group();
		}
		if (result != null) {
			if (objlookup.containsKey(result) && objlookup.get(result) != node) {
				result = result + "_AlloyUID" + id_index++;
			}
		}
		else result = node.eClass().getName() + "_AlloyUID" + id_index++;
		objlookup.put(result, node);
		idlookup.put(node, result);
		return result;
	}
	
	public String getAlloyPreambule() {
		String result = "/****************************************************************/\n";
		result += "abstract sig ResolutionElement {\n";
		result += "	parent: lone CompositeElement\n";
		result += "}\n";
		result += "\n";
		result += "abstract sig CompositeElement extends ResolutionElement {\n";
		result += "	elements : set ResolutionElement\n";
		result += "}\n";
		result += "\n";
		result += "abstract sig ExecutableElement extends ResolutionElement {}\n";
		result += "\n";
		result += "// CompositeElement.elements opposite of ResolutionElement.parent\n";
		result += "fact { all p: CompositeElement, e: p.elements | e.parent = p }\n";
		result += "\n";
		result += "// No cycles\n";
		result += "assert acyclic { no c: CompositeElement | c in c.^elements }\n\n";
		return result;
	}
	
	@Override
	public String visitType(Type node, AlloyGeneratorContext context) {
		String result = getAlloyPreambule();
		String temp1 = "";
		String temp2 = "";
		result += "one sig Type_"+getNodeID(node)+" extends CompositeElement { } { no parent }\n";
		result += "fact { ResolutionElement in Type_"+getNodeID(node)+".*elements }\n";
		
		ArrayList<ExecutablePrimitive> children = selectExecutablePrimitive(node.getBody());
		for(int i = 0 ; i<children.size(); i++) {
			ExecutablePrimitive e = children.get(i);
			temp1 += "e in " + getNodeID(e);
			if (i != children.size() -1) temp1 += " or ";
			temp2 += "fact{ all c: Type_"+getNodeID(node)+" | one e : "+getNodeID(e)+" | e in c.elements }\n";
		}
		if (temp1.length() == 0) temp1 = "no e";
		result += "fact{ all c: Type_"+getNodeID(node)+", e: c.elements | "+temp1+" }\n";
		result += temp2 + "\n";
		result += node.getBody().accept(this, context);
		return result;
	}
	

	public ArrayList<ExecutablePrimitive> selectExecutablePrimitive(EList<VariabilitySpecification> list) {
		ArrayList<ExecutablePrimitive> result = new ArrayList<ExecutablePrimitive>();
		for (VariabilitySpecification e : list) {
			if (e instanceof ExecutablePrimitive) result.add((ExecutablePrimitive)e);
		}
		return result;
	}
	public ArrayList<ExecutablePrimitive> selectExecutablePrimitive(VariabilitySpecification spec) {
		ArrayList<ExecutablePrimitive> result = new ArrayList<ExecutablePrimitive>();
		if (spec instanceof ExecutablePrimitive) result.add((ExecutablePrimitive)spec);
		return result;
	}
	
	public String visitExecutablePrimitive(ExecutablePrimitive node, AlloyGeneratorContext context) {
		return "sig " + getNodeID(node) + " extends ExecutableElement {}\n";
	}
	
	@Override
	public String visitAND_(AND_ node, AlloyGeneratorContext context) {
		return "(" + node.getLeft().accept(this, context) + " and (" + node.getRight().accept(this, context) + ")";
	}

	@Override
	public String visitAssignment(Assignment node, AlloyGeneratorContext context) {
		return visitExecutablePrimitive(node, context);
	}

	@Override
	public String visitCompositeVariability(CompositeVariability node, AlloyGeneratorContext context) {
		String result = "";
		String temp1 = "";
		String temp2 = "";
		result += "sig "+ getNodeID(node) +" extends CompositeElement {}\n";
		ArrayList<ExecutablePrimitive> children = selectExecutablePrimitive(node.getVariabilitySpecification());
		for(int i = 0 ; i<children.size(); i++) {
			ExecutablePrimitive e = children.get(i);
			temp1 += "e in " + getNodeID(e);
			if (i != children.size() -1) temp1 += " or ";
			temp2 += "fact{ all c: "+getNodeID(node)+" | one e : "+getNodeID(e)+" | e in c.elements }\n";
		} 
		if (temp1.length() == 0) temp1 = "no e";
		result += "fact{ all c: "+getNodeID(node)+", e: c.elements | "+temp1+" }\n";
		result += temp2 + "\n";
		for(VariabilitySpecification e : children)
			result += e.accept(this, context);
		return result;
	}

	@Override
	public String visitCVLModel(CVLModel node, AlloyGeneratorContext context) {
		String result = getAlloyPreambule();
		String temp1 = "";
		String temp2 = "";
		result += "one sig CVLModel extends CompositeElement { } { no parent }\n";
		result += "fact { ResolutionElement in CVLModel.*elements }\n";
		
		ArrayList<ExecutablePrimitive> children = selectExecutablePrimitive(node.getVariabilitySpecification());
		for(int i = 0 ; i<children.size(); i++) {
			ExecutablePrimitive e = children.get(i);
			temp1 += "e in " + getNodeID(e);
			if (i != children.size() -1) temp1 += " or ";
			temp2 += "fact{ all c: CVLModel | one e : "+getNodeID(e)+" | e in c.elements }\n";
		} 
		if (temp1.length() == 0) temp1 = "no e";
		result += "fact{ all c: CVLModel, e: c.elements | "+temp1+" }\n";
		result += temp2 + "\n";
		if (node.getVariabilitySpecification() instanceof ExecutablePrimitive)
			result += node.getVariabilitySpecification().accept(this, context);
		return result;
	}

	@Override
	public String visitExecutablePrimitiveTerm(ExecutablePrimitiveTerm node, AlloyGeneratorContext context) {
		return getNodeID(node.getPrimitive());
	}

	@Override
	public String visitFragmentSubstitution(FragmentSubstitution node, AlloyGeneratorContext context) {
		return visitExecutablePrimitive(node, context);
	}

	@Override
	public String visitIMPLIES(IMPLIES node, AlloyGeneratorContext context) {
		return "(" + node.getLeft().accept(this, context) + " implies (" + node.getRight().accept(this, context) + ")";
	}

	@Override
	public String visitInvocation(Invocation node, AlloyGeneratorContext context) {
		return visitExecutablePrimitive(node, context);
	}

	@Override
	public String visitIterator(Iterator node, AlloyGeneratorContext context) {
		String result = "";
		String temp1 = "";
		String temp2 = "";
		result += "sig "+ getNodeID(node) +" extends CompositeElement {}\n";
		result += "{\n";
		result += "  #elements >= " + node.getLowerLimit() + "\n";
		if (node.getUpperLimit()>0)
			result += "  #elements <=" + node.getUpperLimit() + "\n";
		result += "}\n";
		ArrayList<ExecutablePrimitive> children = selectExecutablePrimitive(node.getVariabilitySpecification());
		for(int i = 0 ; i<children.size(); i++) {
			ExecutablePrimitive e = children.get(i);
			temp1 += "e in " + getNodeID(e);
			if (i != children.size() -1) temp1 += " or ";
			temp2 += "fact{ all c: "+getNodeID(node)+" | lone e : "+getNodeID(e)+" | e in c.elements }\n";
		}
		if (temp1.length() == 0) temp1 = "no e";
		result += "fact{ all c: "+getNodeID(node)+", e: c.elements | "+temp1+" }\n";
		if (node.isIsUnique())
			result += temp2 + "\n";
		for(VariabilitySpecification e : children)
			result += e.accept(this, context);
		return result;
	}

	@Override
	public String visitIteratorResolution(IteratorResolution node, AlloyGeneratorContext context) {
		// TODO Auto-generated method stub
		return super.visitIteratorResolution(node, context);
	}

	@Override
	public String visitNOT(NOT node, AlloyGeneratorContext context) {
		// TODO To be fixed when the meta-model will be fixed.
		return "not ";
	}

	@Override
	public String visitOR(OR node, AlloyGeneratorContext context) {
		return "(" + node.getLeft().accept(this, context) + " or (" + node.getRight().accept(this, context) + ")";
	}

	@Override
	public String visitReferenceSubstitution(ReferenceSubstitution node, AlloyGeneratorContext context) {
		return visitExecutablePrimitive(node, context);
	}

	@Override
	public String visitResolutionElement(ResolutionElement node, AlloyGeneratorContext context) {
		// TODO Auto-generated method stub
		return super.visitResolutionElement(node, context);
	}

	@Override
	public String visitResolutionValue(ResolutionValue node, AlloyGeneratorContext context) {
		// TODO Auto-generated method stub
		return super.visitResolutionValue(node, context);
	}

	@Override
	public String visitValueSubstitution(ValueSubstitution node, AlloyGeneratorContext context) {
		return visitExecutablePrimitive(node, context);
	}

}
