package cvl.constraints;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import cvl.CVLElement;
import cvl.CVLModel;
import cvl.ResolutionSpecification;
import cvl.Type;
import edu.mit.csail.sdg.alloy4.Err;
import edu.mit.csail.sdg.alloy4compiler.ast.Module;
import edu.mit.csail.sdg.alloy4compiler.parser.CompUtil;
import edu.mit.csail.sdg.alloy4compiler.translator.A4Options;
import edu.mit.csail.sdg.alloy4compiler.translator.A4Solution;
import edu.mit.csail.sdg.alloy4compiler.translator.TranslateAlloyToKodkod;

public class CVLConstraintValidator {

	public static void runInstancesExistanceChecker(CVLModel model) {
		_runInstancesExistanceChecker(model);
	}

	public static void runInstancesExistanceChecker(Type model) {
		_runInstancesExistanceChecker(model);
	}

	private static void _runInstancesExistanceChecker(CVLElement model) {
		InstancesExistanceChecker.check(model);
	}
	
	public static void runInstancesCounter(CVLModel model) {
		_runInstancesCounter(model);
	}

	public static void runInstancesCounter(Type model) {
		_runInstancesCounter(model);
	}

	private static void _runInstancesCounter(CVLElement model) {
		InstancesCounter.check(model);
	}
	
	public static void runReachabilityAnalyser(CVLModel model) {
		_runReachabilityAnalyser(model);
	}

	public static void runReachabilityAnalyser(Type model) {
		_runReachabilityAnalyser(model);
	}

	private static void _runReachabilityAnalyser(CVLElement model) {
		ReachabilityAnalyser.check(model);
	}
	
	public static void runResolutionChecker(ResolutionSpecification resolution) {
		ResolutionChecker.check(resolution);
	}

	public static A4Solution computeSolutions(String system, int scope) throws IOException, Err {

		// Chooses the Alloy4 options
		A4Options opt = new A4Options();
		opt.solver = A4Options.SatSolver.SAT4J;

		// create a temp file
		File tempfile = File.createTempFile("alloy_system", ".als");
		FileWriter fw = new FileWriter(tempfile);
		fw.write(system);
		fw.write("\n");
		fw.write("run {} for " + scope);
		fw.close();
		tempfile.deleteOnExit();

		Module world = CompUtil.parseEverything_fromFile(null, null, tempfile
				.getAbsolutePath());

		A4Solution solution = TranslateAlloyToKodkod
				.execute_command(null, world.getAllReachableSigs(), world
						.getAllCommands().get(0), opt);

		return solution;

		/*
		 * int i = 0; while(s.satisfiable()) {
		 * 
		 * Iterator<ExprVar> atoms = s.getAllAtoms().iterator();
		 * while(atoms.hasNext()) { ExprVar v = atoms.next(); result +=
		 * v.label.replaceAll("\\$0", "") + " "; } result += "\n";
		 * 
		 * s = s.next(); i++; if (i > 100) break; // just for security } return
		 * result;
		 */
	}


}
