package cvl.constraints;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import cvl.CVLElement;
import cvl.CVLModel;
import edu.mit.csail.sdg.alloy4compiler.translator.A4Solution;

public class InstancesExistanceChecker implements IRunnableWithProgress {

	public static void check(CVLElement model) {
		InstancesExistanceChecker c = new InstancesExistanceChecker();
		c.model = model;
		try {
	      new ProgressMonitorDialog(new Shell()).run(true, false, c);
	    } catch (Exception e) {
	        MessageDialog.openError(new Shell(), "Error", e.getMessage());
	        e.printStackTrace();
	    }
		c.displayResult();
	}
	
	protected CVLElement model;
	protected boolean existInstances = false;
	protected boolean executed = false;
	
	public void displayResult() {
		if (! executed) return;
		if (existInstances) {
			MessageBox messageBox = new MessageBox(new Shell(),
					SWT.ICON_INFORMATION | SWT.OK);
			messageBox.setText("Model Validated Sucessfuly");
			messageBox
					.setMessage("The variability model is consistent and admits at least one valid instance.");
			messageBox.open();
		} else {
			MessageBox messageBox = new MessageBox(new Shell(),
					SWT.ICON_WARNING | SWT.OK);
			messageBox.setText("Model is Inconsistent");
			String msg = "The solver was not able to find any valid instanciation for the variability model.";
			msg += "The variability model probably contains an inconsistency.\n\n";
			msg += "Hint: Check the constraints associated to variability elements and the multiplicities on iterators.";
			messageBox.setMessage(msg);
			messageBox.open();
		}
	}
	
	@Override
	public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
		try {
			
			monitor.beginTask("Checking Existance of Instances", 3);

			monitor.subTask("Generating Formal Specification");
			AlloyGenerator g = new AlloyGenerator();
			String alloy = model.accept(g, new AlloyGeneratorContext());
			monitor.worked(1);

			monitor.subTask("Compiling Alloy Model");
			A4Solution s = CVLConstraintValidator.computeSolutions(alloy, 10);
			monitor.worked(1);

			monitor.subTask("Searching for instances");
			existInstances = s.satisfiable();
			executed = true;
			monitor.worked(1);

		} catch (Exception e) {
			MessageDialog.openError(new Shell(), "Error", e.getMessage());
			e.printStackTrace();
		} finally {
			monitor.done();
		}
	}
}
