package cvl.constraints;

import java.awt.List;
import java.lang.reflect.InvocationTargetException;
import java.util.Hashtable;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import cvl.CVLElement;
import cvl.CVLModel;
import cvl.CVLNamedElement;
import cvl.Type;
import edu.mit.csail.sdg.alloy4compiler.ast.Browsable;
import edu.mit.csail.sdg.alloy4compiler.ast.ExprVar;
import edu.mit.csail.sdg.alloy4compiler.translator.A4Solution;

public class ReachabilityAnalyser implements IRunnableWithProgress {

	public static void check(CVLElement model) {
		ReachabilityAnalyser c = new ReachabilityAnalyser();
		c.model = model;
		try {
	      new ProgressMonitorDialog(new Shell()).run(true, false, c);
	    } catch (Exception e) {
	        MessageDialog.openError(new Shell(), "Error", e.getMessage());
	        e.printStackTrace();
	    }
		c.displayResult();
	}
	
	protected CVLElement model;
	protected int instancesCount = 0;
	protected Hashtable<String, CVLElement> elements = new Hashtable<String, CVLElement>();
	protected boolean executed = false;
	
	public void displayResult() {
		if (! executed) return;
		if (instancesCount > 0) {
			String msg = "";
			if (instancesCount < 251)
				msg += "The analyser has processed " + instancesCount + " resolutions.\n\n";
			else
				msg += "WARNING: The analyser has only processed the first 250 resolutions.\n\n";
			if (elements.isEmpty()) {
				msg += "All the elements of the variability model can be instanciated in at least one resolution.";
				MessageBox messageBox = new MessageBox(new Shell(), SWT.ICON_INFORMATION | SWT.OK);
				messageBox.setText("Model Validated Sucessfuly");
				messageBox.setMessage(msg);
				messageBox.open();
			}
			else {
				MessageBox messageBox = new MessageBox(new Shell(), SWT.ICON_WARNING| SWT.OK);
				messageBox.setText("Some Elements are Unreachable");
				msg += "A total of " + elements.size() + " element(s) of the variability model could not be reached in any resolution.\n";
				msg += "\nUnreachable element(s): \n";
				for (CVLElement e : elements.values()) {
					msg += "  - " + ((CVLNamedElement)e).getName() + "\n";
				}
				messageBox.setMessage(msg);
				messageBox.open();
			}
		} else {
			MessageBox messageBox = new MessageBox(new Shell(),
					SWT.ICON_WARNING | SWT.OK);
			messageBox.setText("Model is Inconsistent");
			String msg = "The solver was not able to find any valid instanciation for the variability model.";
			msg += "The variability model probably contains an inconsistency.\n\n";
			msg += "Hint: Check the constraints associated to variability elements and the multiplicities on iterators.";
			messageBox.setMessage(msg);
			messageBox.open();
		}
	}
	
	@Override
	public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
		try {
			
			monitor.beginTask("Checking Existance of Instances", 3);

			monitor.subTask("Generating Formal Specification");
			AlloyGenerator g = new AlloyGenerator();
			String alloy = model.accept(g, new AlloyGeneratorContext());
			elements.putAll(g.objlookup);
			if (model instanceof Type) {
				elements.remove(g.getNodeID((Type)model));
			}
			monitor.worked(1);

			monitor.subTask("Compiling Alloy Model");
			A4Solution s = CVLConstraintValidator.computeSolutions(alloy, 10);
			monitor.worked(1);

			monitor.subTask("Searching for instances");
			while (s.satisfiable()) {
				instancesCount++;
				
				for (ExprVar e : s.getAllAtoms()) {
					String id = e.toString().split("\\$")[0];
					elements.remove(id);
				}

				s = s.next();
				if (instancesCount == 251) break;
				if (elements.isEmpty()) break;
			}
			executed = true;
			monitor.worked(1);

		} catch (Exception e) {
			MessageDialog.openError(new Shell(), "Error", e.getMessage());
			e.printStackTrace();
		} finally {
			monitor.done();
		}
	}
}
