package cvl.constraints;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import cvl.CVLElement;
import cvl.CVLModel;
import cvl.Invocation;
import cvl.IteratorResolution;
import cvl.ResolutionElement;
import cvl.ResolutionSpecification;
import edu.mit.csail.sdg.alloy4compiler.translator.A4Solution;

public class ResolutionChecker implements IRunnableWithProgress {

	public static void check(ResolutionSpecification resolution) {
		ResolutionChecker c = new ResolutionChecker();
		c.resolution = resolution;
		
		if (resolution instanceof ResolutionElement) {
			ResolutionElement e = (ResolutionElement)resolution;
			if (e.getElement() instanceof Invocation) {
				Invocation i = (Invocation)e.getElement();
				c.model = i.getType();
			}
			else {
				MessageDialog.openError(new Shell(), "Error", "The resolution checker should only be called from the root" +
						" resolution or from the resolution of an invocation");
				return;
			}
		}
		if (resolution instanceof IteratorResolution) {
			IteratorResolution r = (IteratorResolution)resolution;
			if (r.eContainer() != null && r.eContainer() instanceof CVLModel) {
				c.model = (CVLModel)r.eContainer();
			}
			else {
				MessageDialog.openError(new Shell(), "Error", "The resolution checker should only be called from the root" +
				" resolution or from the resolution of an invocation");
				return;
			}
		}
		
		
		try {
	      new ProgressMonitorDialog(new Shell()).run(true, false, c);
	    } catch (Exception e) {
	        MessageDialog.openError(new Shell(), "Error", e.getMessage());
	        e.printStackTrace();
	    }
		c.displayResult();
	}
	
	protected CVLElement model;
	protected ResolutionSpecification resolution;
	protected boolean existInstances = false;
	protected boolean executed = false;
	
	public void displayResult() {
		if (! executed) return;
		if (existInstances) {
			MessageBox messageBox = new MessageBox(new Shell(),
					SWT.ICON_INFORMATION | SWT.OK);
			messageBox.setText("Model Validated Sucessfuly");
			messageBox
					.setMessage("The resolution model is consistent.");
			messageBox.open();
		} else {
			MessageBox messageBox = new MessageBox(new Shell(),
					SWT.ICON_WARNING | SWT.OK);
			messageBox.setText("Model is Inconsistent");
			String msg = "The solver was not able to find a valid instanciation for the variability model which corresponds to this resolution.";
			msg += "Either the variability model contains an inconsistency or the resolution model is invalid.\n\n";
			msg += "Hint: Check that the resolution model follows the structure of the variability model and its associated constraints.";
			messageBox.setMessage(msg);
			messageBox.open();
		}
	}
	
	@Override
	public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
		try {
			
			monitor.beginTask("Checking Resolution Model Consistency", 4);

			monitor.subTask("Generating Formal Specification");
			AlloyGenerator g = new AlloyGenerator();
			String alloy = model.accept(g, new AlloyGeneratorContext());
			monitor.worked(1);
			
			monitor.subTask("Generating Resolution");

			alloy += resolution.accept(g, new AlloyGeneratorContext());
			monitor.worked(1);

			monitor.subTask("Compiling Alloy Model");
			A4Solution s = CVLConstraintValidator.computeSolutions(alloy, 10);
			monitor.worked(1);

			monitor.subTask("Cheching consistency");
			existInstances = s.satisfiable();
			executed = true;
			monitor.worked(1);

		} catch (Exception e) {
			//MessageDialog.openError(new Shell(), "Error", e.getMessage());
			e.printStackTrace();
		} finally {
			monitor.done();
		}
	}
}
