package cvl.editor;

import org.eclipse.emf.ecore.EObject;

import cvl.*;
import cvl.interop.ICVLEnabledEditor;
import cvl.util.CVLUtil;
import cvl.visitors.Visitable;
import cvl.visitors.impl.TopDownVisitorImpl;

public class BaseModelHighlighter extends TopDownVisitorImpl<CVLTreeEditor, Object> {

	public void highlightForObject(EObject o, CVLTreeEditor editor) {
		editor.clearHighlighting();
		if (o instanceof Visitable) {
			try {
				((Visitable) o).accept(this, editor);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	@Override
	public Object visitCVLElement(CVLElement o, CVLTreeEditor editor) {
		return null;
	}

	@Override
	public Object visitCVLModel(CVLModel o, CVLTreeEditor editor) {
		return null;
	}



	@Override
	public Object visitFromBinding(FromBinding o, CVLTreeEditor editor) {
		if (o.getFromPlacement() != null) o.getFromPlacement().accept(this, editor);
		if(o.getFromReplacement() != null) o.getFromReplacement().accept(this, editor);
		return null;
	}

	@Override
	public Object visitFromPlacement(FromPlacement o, CVLTreeEditor editor) {
		for (Object obj : o.getOutsideBoundaryElement())
			editor.highlightObject(obj, ICVLEnabledEditor.HL_PLACEMENT_OUT);
		return super.visitFromPlacement(o, editor);
	}

	@Override
	public Object visitFromReplacement(FromReplacement o, CVLTreeEditor editor) {
		editor.highlightObject(o.getInsideBoundaryElement(), ICVLEnabledEditor.HL_REPLACEMENT);
		for (Object obj : o.getOutsideBoundaryElement())
			editor.highlightObject(obj, ICVLEnabledEditor.HL_REPLACEMENT_OUT);
		return super.visitFromReplacement(o, editor);
	}

	@Override
	public Object visitPlacementFragment(PlacementFragment o, CVLTreeEditor editor) {
		for(EObject obj : CVLUtil.getPlacementFragmentContent(o)) {
			editor.highlightObject(obj, ICVLEnabledEditor.HL_PLACEMENT);
		}
		return super.visitPlacementFragment(o, editor);
	}

	@Override
	public Object visitReferenceSubstitution(ReferenceSubstitution node, CVLTreeEditor context) {
		if (node.getPlacement() != null) node.getPlacement().accept(this, context);
		if (node.getReplacement() != null) node.getReplacement().accept(this, context);
		return null;
	}

	@Override
	public Object visitReplacementFragment(ReplacementFragment o, CVLTreeEditor editor) {
		for(EObject obj : CVLUtil.getReplacementFragmentContent(o)) {
			editor.highlightObject(obj, ICVLEnabledEditor.HL_REPLACEMENT);
		}
		
		for(ReplacementBoundaryElement e : o.getBoundaryElement())
			e.accept(this, editor);
		
		return null;
	}


	@Override
	public Object visitToBinding(ToBinding o, CVLTreeEditor editor) {
		if (o.getToPlacement() != null) o.getToPlacement().accept(this, editor);
		if (o.getToReplacement() != null) o.getToReplacement().accept(this, editor);
		return null;
	}

	@Override
	public Object visitToPlacement(ToPlacement o, CVLTreeEditor editor) {
		editor.highlightObject(o.getOutsideBoundaryElement(), ICVLEnabledEditor.HL_PLACEMENT_OUT);
		for (EObject e : o.getInsideBoundaryElement()) {
			editor.highlightObject(e, ICVLEnabledEditor.HL_PLACEMENT);
		}
		return super.visitToPlacement(o, editor);
	}

	@Override
	public Object visitToReplacement(ToReplacement o, CVLTreeEditor editor) {
		for (EObject e : o.getInsideBoundaryElement()) {
			editor.highlightObject(e, ICVLEnabledEditor.HL_REPLACEMENT);
		}
		return super.visitToReplacement(o, editor);
	}


	@Override
	public Object visitValueSubstitution(ValueSubstitution node, CVLTreeEditor context) {
		if (node.getPlacement() != null) node.getPlacement().accept(this, context);
		if (node.getReplacement() != null) node.getReplacement().accept(this, context);
		return null;
	}

	@Override
	public Object visitFragmentSubstitution(FragmentSubstitution node, CVLTreeEditor context) {
		if (node.getPlacement() != null) node.getPlacement().accept(this, context);
		if (node.getReplacement() != null) node.getReplacement().accept(this, context);
		return super.visitFragmentSubstitution(node, context);
	}

	@Override
	public Object visitPlacementObject(PlacementObject node, CVLTreeEditor context) {
		context.highlightObject(node.getReferredObject(), ICVLEnabledEditor.HL_PLACEMENT);
		context.highlightObject(node.getTargetObject(), ICVLEnabledEditor.HL_PLACEMENT_OUT);
		return null;
	}

	@Override
	public Object visitPlacementValue(PlacementValue node, CVLTreeEditor context) {
		context.highlightObject(node.getTargetObject(), ICVLEnabledEditor.HL_PLACEMENT_OUT);
		return super.visitPlacementValue(node, context);
	}

	@Override
	public Object visitReplacementFragmentReference(ReplacementFragmentReference node, CVLTreeEditor context) {
		if (node.getRfRef() != null) node.getRfRef().accept(this, context);
		return super.visitReplacementFragmentReference(node, context);
	}

	@Override
	public Object visitReplacementObject(ReplacementObject node, CVLTreeEditor context) {
		context.highlightObject(node.getTargetObject(), ICVLEnabledEditor.HL_REPLACEMENT);
		return super.visitReplacementObject(node, context);
	}

	@Override
	public Object visitReplacementValue(ReplacementValue node, CVLTreeEditor context) {
		return super.visitReplacementValue(node, context);
	}

	@Override
	public Object visitCompositeVariability(CompositeVariability node,
			CVLTreeEditor context) {
		return null;
	}

	@Override
	public Object visitIterator(Iterator node, CVLTreeEditor context) {
		return null;
	}
	
	
}
