package cvl.editor;

import org.eclipse.emf.ecore.impl.EObjectImpl;

import cvl.AND_;
import cvl.Assignment;
import cvl.CVLElement;
import cvl.CVLModel;
import cvl.CompositeVariability;
import cvl.Constraint;
import cvl.ExecutablePrimitiveTerm;
import cvl.FragmentSubstitution;
import cvl.FromBinding;
import cvl.FromPlacement;
import cvl.FromReplacement;
import cvl.IMPLIES;
import cvl.Invocation;
import cvl.Iterator;
import cvl.IteratorResolution;
import cvl.NOT;
import cvl.OR;
import cvl.PlacementFragment;
import cvl.PlacementObject;
import cvl.PlacementValue;
import cvl.ReferenceSubstitution;
import cvl.ReplacementFragment;
import cvl.ReplacementFragmentReference;
import cvl.ReplacementObject;
import cvl.ReplacementValue;
import cvl.ResolutionElement;
import cvl.ResolutionValue;
import cvl.ToBinding;
import cvl.ToPlacement;
import cvl.ToReplacement;
import cvl.Type;
import cvl.ValueSubstitution;
import cvl.visitors.TopDownVisitor;

public class CVLTextLabelProvider extends EObjectImpl implements TopDownVisitor<String, String> {

	private static CVLTextLabelProvider instance = new CVLTextLabelProvider();
	public static CVLTextLabelProvider getInstance() {
		return instance;
	}
	
	private CVLTextLabelProvider() {}
	
	@Override
	public String visitAssignment(Assignment node, String context) {
		return node.getTarget().accept(this, null) + " := " + node.getValue().accept(this, null);
	}

	@Override
	public String visitCVLElement(CVLElement node, String context) {
		return null;
	}

	@Override
	public String visitCVLModel(CVLModel node, String context) {
		return node.getName();
	}

	@Override
	public String visitCompositeVariability(CompositeVariability node, String context) {
		String result = node.getName();
		if (result == null) result = "{...}";
		return result;
	}

	@Override
	public String visitFragmentSubstitution(FragmentSubstitution node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitFromBinding(FromBinding node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitFromPlacement(FromPlacement node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitFromReplacement(FromReplacement node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitInvocation(Invocation node, String context) {
		if (node.getRfRef() != null)
			return node.getRfRef().accept(this, null) + " := " + node.getType().getName() + "()";
		else return node.getType().getName() + "()";
	}

	@Override
	public String visitIterator(Iterator node, String context) {
		return node.getName() + " [" + node.getLowerLimit() + ".." + node.getUpperLimit() + "]";
	}

	@Override
	public String visitIteratorResolution(IteratorResolution node, String context) {
		return " -> " + node.getIterator().getName();
	}

	@Override
	public String visitPlacementFragment(PlacementFragment node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitPlacementObject(PlacementObject node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitPlacementValue(PlacementValue node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitReferenceSubstitution(ReferenceSubstitution node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitReplacementFragment(ReplacementFragment node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitReplacementFragmentReference(ReplacementFragmentReference node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitReplacementObject(ReplacementObject node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitReplacementValue(ReplacementValue node, String context) {
		return node.getValue();
	}

	@Override
	public String visitResolutionElement(ResolutionElement node, String context) {
		String result = node.getName();
		if (result == null) result = "";
		return result + " -> " + node.getElement().accept(this, null);
	}

	@Override
	public String visitToBinding(ToBinding node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitToPlacement(ToPlacement node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitToReplacement(ToReplacement node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitType(Type node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitValueSubstitution(ValueSubstitution node, String context) {
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitAND_(AND_ node, String context) {
		// TODO Auto-generated method stub
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitConstraint(Constraint node, String context) {
		// TODO Auto-generated method stub
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitExecutablePrimitiveTerm(ExecutablePrimitiveTerm node, String context) {
		// TODO Auto-generated method stub
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitIMPLIES(IMPLIES node, String context) {
		// TODO Auto-generated method stub
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitNOT(NOT node, String context) {
		// TODO Auto-generated method stub
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitOR(OR node, String context) {
		// TODO Auto-generated method stub
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}

	@Override
	public String visitResolutionValue(ResolutionValue node, String context) {
		// TODO Auto-generated method stub
		String result = node.getName();
		if (result == null) result = "No Name";
		return result;
	}
}
