package cvl.editor;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.common.ui.ViewerPane;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.command.RemoveCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.ui.celleditor.AdapterFactoryTreeEditor;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import cvl.BoundaryElementBinding;
import cvl.CVLElement;
import cvl.CVLModel;
import cvl.CVLNamedElement;
import cvl.CompositeVariability;
import cvl.CvlPackage;
import cvl.FragmentSubstitution;
import cvl.Invocation;
import cvl.IteratorResolution;
import cvl.PlacementFragment;
import cvl.PlacementObject;
import cvl.PlacementValue;
import cvl.ReplacementFragment;
import cvl.ReplacementFragmentBinding;
import cvl.ReplacementObject;
import cvl.ResolutionElement;
import cvl.ResolutionSpecification;
import cvl.Type;
import cvl.constraints.AlloyGenerator;
import cvl.constraints.AlloyGeneratorContext;
import cvl.constraints.CVLConstraintValidator;
import cvl.interop.ICVLEnabledEditor;
import cvl.presentation.CvlEditor;
import cvl.presentation.CvlEditor.ReverseAdapterFactoryContentProvider;
import cvl.util.CVLUtil;


public class CVLTreeEditor extends CvlEditor {
	
	//public CVLLabelProvider outlineLabelProvider;
	public BaseModelHighlighter highlightingProvider = new BaseModelHighlighter();
	
	FragmentBindingEditorPane fragmentBindingEditorPane;
	int fragmentBindingEditorPanePage;
	
	protected ComposedAdapterFactory getAdaptedFactory() {
		return adapterFactory;
	}
	
	protected Composite getViewerPaneContainer() {
		return getContainer();
	}
	
	public void handleContentOutlineSelection(ISelection selection) {
		// Do nothing
	}
	
	/*
	public IContentOutlinePage getContentOutlinePage() {
		if (contentOutlinePage == null) {
			// The content outline is just a tree.
			//
			class MyContentOutlinePage extends ContentOutlinePage {
				@Override
				public void createControl(Composite parent) {
					super.createControl(parent);
					contentOutlineViewer = getTreeViewer();
					//contentOutlineViewer.addSelectionChangedListener(this);

					// Set up the tree viewer.
					//
					contentOutlineViewer.setContentProvider(new AdapterFactoryContentProvider(adapterFactory));
					outlineLabelProvider = new CVLLabelProvider(new AdapterFactoryLabelProvider(adapterFactory));
					contentOutlineViewer.setLabelProvider(outlineLabelProvider);
					contentOutlineViewer.setInput(editingDomain.getResourceSet());

					// Make sure our popups work.
					//
					// createContextMenuFor(contentOutlineViewer);

					if (!editingDomain.getResourceSet().getResources().isEmpty()) {
					  // Select the root object in the view.
					  //
					  //contentOutlineViewer.setSelection(new StructuredSelection(editingDomain.getResourceSet().getResources().get(0)), true);
					}
				}

				@Override
				public void makeContributions(IMenuManager menuManager, IToolBarManager toolBarManager, IStatusLineManager statusLineManager) {
					super.makeContributions(menuManager, toolBarManager, statusLineManager);
					contentOutlineStatusLineManager = statusLineManager;
				}

				@Override
				public void setActionBars(IActionBars actionBars) {
					super.setActionBars(actionBars);
					getActionBarContributor().shareGlobalActions(this, actionBars);
				}
			}

			contentOutlinePage = new MyContentOutlinePage();
			/*
			// Listen to selection so that we can handle it is a special way.
			//*
			contentOutlinePage.addSelectionChangedListener
				(new ISelectionChangedListener() {
					 // This ensures that we handle selections correctly.
					 //
					 public void selectionChanged(SelectionChangedEvent event) {
						 handleContentOutlineSelection(event.getSelection());
					 }
				 });
			*//*
		}

		return contentOutlinePage;
	}
	*/
	
	
	public void menuAboutToShow(IMenuManager menuManager) {
		super.menuAboutToShow(menuManager);

		if (editorSelection != null && editorSelection instanceof TreeSelection) {
			TreeSelection s = (TreeSelection)editorSelection;

			if (s.size() == 1 && s.getFirstElement() instanceof CompositeVariability) {
				final CompositeVariability block = (CompositeVariability) s.getFirstElement();


				menuManager.insertBefore("edit", new Action("Create Placement Fragment From Selection") {
					@Override
					public void run() {
						System.out.println("Create Placement Fragment for " + block);
						ArrayList<EObject> objects = getSelectedObjects();
						if (objects != null && objects.size() > 0) { 
							PlacementFragment pf = CVLUtil.createPlacementFragment(objects);
							AddCommand cmd = new AddCommand(getEditingDomain(), block, CvlPackage.eINSTANCE.getCompositeVariability_VariabilitySpecification(), pf);
							getEditingDomain().getCommandStack().execute(cmd);
						}
					}
				});


				menuManager.insertBefore("edit", new Action("Create Replacement Fragment From Selection") {
					@Override
					public void run() {
						System.out.println("Add Replacement Fragment for " + block);
						ArrayList<EObject> objects = getSelectedObjects();
						if (objects != null && objects.size() > 0) { 
							ReplacementFragment rf = CVLUtil.createReplacementFragment(objects);
							AddCommand cmd = new AddCommand(getEditingDomain(), block, CvlPackage.eINSTANCE.getCompositeVariability_VariabilitySpecification(), rf);
							getEditingDomain().getCommandStack().execute(cmd);
						}
					}
				});
				
				menuManager.insertBefore("edit", new Action("Create Placement Fragment From Selection with Contained Objects") {
					@Override
					public void run() {
						System.out.println("Create Placement Fragment for " + block);
						ArrayList<EObject> objects = getSelectedObjects();
						objects = CVLUtil.includeContainment(objects);  //also include contained objects
						if (objects != null && objects.size() > 0) { 
							PlacementFragment pf = CVLUtil.createPlacementFragment(objects);
							AddCommand cmd = new AddCommand(getEditingDomain(), block, CvlPackage.eINSTANCE.getCompositeVariability_VariabilitySpecification(), pf);
							getEditingDomain().getCommandStack().execute(cmd);
						}
					}
				});
				
				menuManager.insertBefore("edit", new Action("Create Replacement Fragment From Selection with Contained Objects") {
					@Override
					public void run() {
						System.out.println("Add Replacement Fragment for " + block);
						ArrayList<EObject> objects = getSelectedObjects();
						objects = CVLUtil.includeContainment(objects);  //also include contained objects
						if (objects != null && objects.size() > 0) { 
							ReplacementFragment rf = CVLUtil.createReplacementFragment(objects);
							AddCommand cmd = new AddCommand(getEditingDomain(), block, CvlPackage.eINSTANCE.getCompositeVariability_VariabilitySpecification(), rf);
							getEditingDomain().getCommandStack().execute(cmd);
						}
					}
				});

			}
			
			if (s.size() == 1 && s.getFirstElement() instanceof PlacementValue) {
				final PlacementValue pv = (PlacementValue) s.getFirstElement();
				menuManager.insertBefore("edit", new Action("Set Placement Target From Selection") {
					@Override
					public void run() {
						System.out.println("Set Placement Target From Selection for " + pv);
						ArrayList<EObject> objects = getSelectedObjects();
						if (objects != null && objects.size() > 0) { 
							SetCommand cmd = new SetCommand(getEditingDomain(), pv, CvlPackage.eINSTANCE.getPlacementValue_TargetObject(), objects.get(0));
							getEditingDomain().getCommandStack().execute(cmd);
						}
					}
				});
			}
			
			if (s.size() == 1 && s.getFirstElement() instanceof PlacementObject) {
				final PlacementObject pv = (PlacementObject) s.getFirstElement();
				menuManager.insertBefore("edit", new Action("Set Placement Target From Selection") {
					@Override
					public void run() {
						System.out.println("Set Placement Target From Selection for " + pv);
						ArrayList<EObject> objects = getSelectedObjects();
						if (objects != null && objects.size() > 0) { 
							SetCommand cmd = new SetCommand(getEditingDomain(), pv, CvlPackage.eINSTANCE.getPlacementObject_TargetObject(), objects.get(0));
							getEditingDomain().getCommandStack().execute(cmd);
						}
					}
				});
				
				menuManager.insertBefore("edit", new Action("Set Referred Object From Selection") {
					@Override
					public void run() {
						System.out.println("Set Referred Object From Selection for " + pv);
						ArrayList<EObject> objects = getSelectedObjects();
						if (objects != null && objects.size() > 0) { 
							SetCommand cmd = new SetCommand(getEditingDomain(), pv, CvlPackage.eINSTANCE.getPlacementObject_ReferredObject(), objects.get(0));
							getEditingDomain().getCommandStack().execute(cmd);
						}
					}
				});
			}
			
			if (s.size() == 1 && s.getFirstElement() instanceof ReplacementObject) {
				final ReplacementObject pv = (ReplacementObject) s.getFirstElement();
				menuManager.insertBefore("edit", new Action("Set Replacement Target From Selection") {
					@Override
					public void run() {
						System.out.println("Set Replacement Target From Selection for " + pv);
						ArrayList<EObject> objects = getSelectedObjects();
						if (objects != null && objects.size() > 0) { 
							SetCommand cmd = new SetCommand(getEditingDomain(), pv, CvlPackage.eINSTANCE.getReplacementObject_TargetObject(), objects.get(0));
							getEditingDomain().getCommandStack().execute(cmd);
						}
					}
				});
			}
			
			
			else if (s.size() == 1 && s.getFirstElement() instanceof ReplacementFragment) {
				final ReplacementFragment rfrag = (ReplacementFragment) s.getFirstElement();
				
					

					menuManager.insertBefore("edit", new Action("Create Placement Fragment From Selection") {
						@Override
						public void run() {
							System.out.println("Create Placement Fragment for " + rfrag);
							ArrayList<EObject> objects = getSelectedObjects();
							if (objects != null && objects.size() > 0) {
								PlacementFragment pf = CVLUtil.createPlacementFragment(objects);
								
								AddCommand cmd = new AddCommand(getEditingDomain(), rfrag, CvlPackage.eINSTANCE.getReplacementFragment_Placements(), pf);
								getEditingDomain().getCommandStack().execute(cmd);
							}
						}
					});
					
					menuManager.insertBefore("edit", new Action("Create Placement Fragment From Selection with Contained Objects") {
						@Override
						public void run() {
							System.out.println("Create Placement Fragment for " + rfrag);
							ArrayList<EObject> objects = getSelectedObjects();
							objects = CVLUtil.includeContainment(objects);  //also include contained objects
							if (objects != null && objects.size() > 0) {
								PlacementFragment pf = CVLUtil.createPlacementFragment(objects);
								
								AddCommand cmd = new AddCommand(getEditingDomain(), rfrag, CvlPackage.eINSTANCE.getReplacementFragment_Placements(), pf);
								getEditingDomain().getCommandStack().execute(cmd);
							}
						}
					});
				
			}
			
			else if (s.size() == 1 && s.getFirstElement() instanceof FragmentSubstitution) {
				final FragmentSubstitution substitution = (FragmentSubstitution) s.getFirstElement();
				if(substitution.getPlacement()!= null && CVLUtil.getConcreteReplacement(substitution.getReplacement()) != null) {
					menuManager.insertBefore("edit", new Action("Create Default Replacement Bindings") {
						@Override
						public void run() {
							ArrayList<BoundaryElementBinding> bs = CVLUtil.createDefaultBindings(substitution.getPlacement(), CVLUtil.getConcreteReplacement(substitution.getReplacement()));
							CompoundCommand cmds = new CompoundCommand();
							// get rid of existing stuff
							cmds.append(new RemoveCommand(getEditingDomain(), substitution.getBoundaryElementBinding(), substitution.getBoundaryElementBinding()));
							// add the new stuff
							cmds.append(new AddCommand(getEditingDomain(), substitution.getBoundaryElementBinding(), bs));
							// actually do it
							getEditingDomain().getCommandStack().execute(cmds);
						}
					});
					menuManager.insertBefore("edit", new Action("Open Bindings Editor") {
						@Override
						public void run() {
							if (fragmentBindingEditorPane != null) {
								fragmentBindingEditorPane.treeViewerWithColumns.setInput(substitution);
								setActivePage(fragmentBindingEditorPanePage);
							}
							
						}
					});
				}
			}
			else if (s.size() == 1 && ((s.getFirstElement() instanceof Type))) {
				final Type elem = (Type) s.getFirstElement();
				
					
					menuManager.insertBefore("edit", new Action("Export alloy formal specification") {
						@Override
						public void run() {
							
							AlloyGenerator gen = new AlloyGenerator();
							String spec = elem.accept(gen, new AlloyGeneratorContext());
							
							System.out.println("Alloy Specification for " + elem.getName() + ":\n");
							System.out.println(spec);
						}
					});
					
					menuManager.insertBefore("edit", new Action("Check the existance of instances") {
						@Override
						public void run() {
							CVLConstraintValidator.runInstancesExistanceChecker(elem);
						}
					});
					menuManager.insertBefore("edit", new Action("Check the number of instances") {
						@Override
						public void run() {
							CVLConstraintValidator.runInstancesCounter(elem);
						}
					});
					menuManager.insertBefore("edit", new Action("Check the reachability of elements") {
						@Override
						public void run() {
							CVLConstraintValidator.runReachabilityAnalyser(elem);
						}
					});
				
			}
			else if (s.size() == 1 && (s.getFirstElement() instanceof CVLModel )) {
				final CVLModel elem = (CVLModel) s.getFirstElement();
				
					
					menuManager.insertBefore("edit", new Action("Export alloy formal specification") {
						@Override
						public void run() {
							
							AlloyGenerator gen = new AlloyGenerator();
							String spec = elem.accept(gen, new AlloyGeneratorContext());
							
							System.out.println("Alloy Specification for " + elem.getName() + ":\n");
							System.out.println(spec);
						}
					});
					
					menuManager.insertBefore("edit", new Action("Check the existance of instances") {
						@Override
						public void run() {
							CVLConstraintValidator.runInstancesExistanceChecker(elem);
						}
					});
					menuManager.insertBefore("edit", new Action("Check the number of instances") {
						@Override
						public void run() {
							CVLConstraintValidator.runInstancesCounter(elem);
						}
					});
					menuManager.insertBefore("edit", new Action("Check the reachability of elements") {
						@Override
						public void run() {
							CVLConstraintValidator.runReachabilityAnalyser(elem);
						}
					});
				
			}
			else if (s.size() == 1 && (s.getFirstElement() instanceof ResolutionSpecification )) {
				final ResolutionSpecification resolution = (ResolutionSpecification) s.getFirstElement();
				
					if ((resolution instanceof IteratorResolution && resolution.eContainer() instanceof CVLModel) || 
					   (resolution instanceof ResolutionElement && ((ResolutionElement)resolution).getElement() instanceof Invocation)) {
					
						menuManager.insertBefore("edit", new Action("Export alloy formal specification") {
							@Override
							public void run() {
								
								AlloyGenerator gen = new AlloyGenerator();
								String spec = resolution.accept(gen, new AlloyGeneratorContext());
								
								System.out.println("Alloy Specification for " + resolution.getName() + ":\n");
								System.out.println(spec);
							}
						});
						
						menuManager.insertBefore("edit", new Action("Validate resolution") {
							@Override
							public void run() {
								CVLConstraintValidator.runResolutionChecker(resolution);
							}
						});
					}
			}
			
		}
		
	}
	
	public void setSelection(ISelection selection) {
		super.setSelection(selection);
		if (editorSelection != null && editorSelection instanceof TreeSelection) {
			TreeSelection s = (TreeSelection)editorSelection;
			if (s.size() == 1 && s.getFirstElement() instanceof EObject) {
				EObject selected = (EObject)s.getFirstElement();
				highlightingProvider.highlightForObject(selected, this);
				//if (contentOutlineViewer != null ) contentOutlineViewer.refresh();
			}
		}
	}
	
	@Override
	public void createPages() {
		// Creates the model from the editor input
		//
		createModel();

		// Only creates the other pages if there is something that can be edited
		//
		if (!getEditingDomain().getResourceSet().getResources().isEmpty()) {
			// Create a page for the selection tree view.
			//
			{
				ViewerPane viewerPane =
					new ViewerPane(getSite().getPage(), CVLTreeEditor.this) {
						@Override
						public Viewer createViewer(Composite composite) {
							Tree tree = new Tree(composite, SWT.MULTI);
							TreeViewer newTreeViewer = new TreeViewer(tree);
							return newTreeViewer;
						}
						@Override
						public void requestActivation() {
							super.requestActivation();
							setCurrentViewerPane(this);
						}
					};
				viewerPane.createControl(getContainer());

				selectionViewer = (TreeViewer)viewerPane.getViewer();
				selectionViewer.setContentProvider(new AdapterFactoryContentProvider(adapterFactory));

				selectionViewer.setLabelProvider(new CVLLabelProvider(new AdapterFactoryLabelProvider(adapterFactory)));
				selectionViewer.setInput(editingDomain.getResourceSet());
				selectionViewer.setSelection(new StructuredSelection(editingDomain.getResourceSet().getResources().get(0)), true);
				viewerPane.setTitle(editingDomain.getResourceSet());

				new AdapterFactoryTreeEditor(selectionViewer.getTree(), adapterFactory);

				createContextMenuFor(selectionViewer);
				int pageIndex = addPage(viewerPane.getControl());
				setPageText(pageIndex, "CVL Tree Editor");
			}

			
			getSite().getShell().getDisplay().asyncExec
				(new Runnable() {
					 public void run() {
						 setActivePage(0);
					 }
				 });
		}
		
		// This is the page for the binding editor
		//
		
		{
			fragmentBindingEditorPane = new FragmentBindingEditorPane(this, getSite().getPage(), CVLTreeEditor.this);
			createContextMenuFor(fragmentBindingEditorPane.getStructuredviewer());
			fragmentBindingEditorPanePage = addPage(fragmentBindingEditorPane.getControl());
			setPageText(fragmentBindingEditorPanePage, "Fragment Binding Editor");
		}
		
		// Ensures that this editor will only display the page's tab
		// area if there are more than one page
		//
		getContainer().addControlListener
			(new ControlAdapter() {
				boolean guard = false;
				@Override
				public void controlResized(ControlEvent event) {
					if (!guard) {
						guard = true;
						hideTabs();
						guard = false;
					}
				}
			 });

		getSite().getShell().getDisplay().asyncExec
			(new Runnable() {
				 public void run() {
					 updateProblemIndication();
				 }
			 });
	}


	
	
	protected ArrayList<ICVLEnabledEditor> getCVLEnabledEditors() {
		ArrayList<ICVLEnabledEditor> result = new ArrayList<ICVLEnabledEditor>();
		// Get all CVL Enabled editors currently opened
		IWorkbenchWindow window=PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		IWorkbenchPage page = window.getActivePage();
		for (IEditorReference e : page.getEditorReferences()) {
			IWorkbenchPart p = e.getPart(false);
			if (p instanceof ICVLEnabledEditor && p != this)
				result.add((ICVLEnabledEditor)p);
		}

		return result;
	}
	

	public void clearHighlighting() {
		for(ICVLEnabledEditor e : getCVLEnabledEditors()) e.clearHighlighting();
	}


	public ArrayList<EObject> getSelectedObjects() {
		ArrayList<EObject> selectedObjects=null;
		boolean foundEditor=false;
		for(ICVLEnabledEditor e : getCVLEnabledEditors()) {
			foundEditor = true;
			if (e.getSelectedObjects().size() > 0) {
				if (selectedObjects != null){
					MessageDialog.openError(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), "CVL Editor", "Selected objects found in more than one CVL enabled editor. Please make sure to select objects in only one editor.");
					return null;
				}
				selectedObjects = resolveLocalObjects(e.getSelectedObjects());
			}
		}
		
		if (selectedObjects == null){
			if (foundEditor){
				MessageDialog.openError(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), "CVL Editor", "No objects selected. Please make a selection in a CVL enabled editor.");
			}else{
				MessageDialog.openError(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), "CVL Editor", "No CVL enabled editors open. Please make a selection in a CVL enabled editor.");
			}
		}
		
		return selectedObjects;
	}
	
	public ArrayList<EObject> resolveLocalObjects(ArrayList<Object> objects) {
		ArrayList<EObject> result = new ArrayList<EObject>();
		for(Object o : objects) {
			if (o instanceof EObject) {
				EObject resolved = getEditingDomain().getResourceSet().getEObject(EcoreUtil.getURI((EObject)o), true);
				if (resolved != null) {
					result.add(resolved);
				}
			}
		}

		// For debug
		if (result.size() != objects.size()) 
			System.err.println("resolveLocalObjects : some object were not resolved");
		return result;
	}


	public void highlightObject(Object o, int type) {
		if(! (o instanceof EObject)) return;
		for(ICVLEnabledEditor e : getCVLEnabledEditors()) e.highlightObject(ICVLEnabledEditor.IDProvider.getObjectId((EObject)o), type);
	}


	public void selectObjects(ArrayList<Object> objects) {
		// TODO Auto-generated method stub
		
	}
}
