package cvl.editor;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.TreeViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;

import cvl.BoundaryElementBinding;
import cvl.CvlPackage;
import cvl.FragmentSubstitution;
import cvl.FromBinding;
import cvl.FromPlacement;
import cvl.FromReplacement;
import cvl.ReplacementFragment;
import cvl.ReplacementFragmentBinding;
import cvl.ToBinding;
import cvl.ToPlacement;
import cvl.ToReplacement;
import cvl.util.CVLUtil;

public class FragmentBindingEditorPane extends TableEditorPane {

	
	public FragmentBindingEditorPane(CVLTreeEditor editor, IWorkbenchPage page, IWorkbenchPart part) {
		super(editor, page, part);
	}
	
	@Override
	protected void initializeTableTreeViewer() {
		Tree tree = treeViewerWithColumns.getTree();
		tree.setLayoutData(new FillLayout());
		tree.setHeaderVisible(true);
		tree.setLinesVisible(true);

		createColumns(treeViewerWithColumns);
		
		TreeViewerColumn viewerColumn;
		TreeColumn column;

		viewerColumn = new TreeViewerColumn(treeViewerWithColumns, SWT.LEAD);
		column = viewerColumn.getColumn();
		columns.add(column);
		column.setText("[Object].property");
		// column.setToolTipText(descriptor.description);
		column.setAlignment(SWT.RIGHT);
		column.setWidth(200);
		viewerColumn.setLabelProvider(new LeftBindingColumnLabelProvider());
		//viewerColumn.setEditingSupport(new PriorityRuleEditingSupport(viewer, p));
		
		viewerColumn = new TreeViewerColumn(treeViewerWithColumns, SWT.LEAD);
		column = viewerColumn.getColumn();
		columns.add(column);
		column.setText("");
		// column.setToolTipText(descriptor.description);
		column.setAlignment(SWT.CENTER);
		column.setWidth(25);
		viewerColumn.setLabelProvider(new MiddleBindingColumnLabelProvider());
		//viewerColumn.setEditingSupport(new PriorityRuleEditingSupport(viewer, p));
		
		viewerColumn = new TreeViewerColumn(treeViewerWithColumns, SWT.LEAD);
		column = viewerColumn.getColumn();
		columns.add(column);
		column.setText("Values");
		// column.setToolTipText(descriptor.description);
		column.setAlignment(SWT.LEFT);
		column.setWidth(200);
		viewerColumn.setLabelProvider(new RightBindingColumnLabelProvider());
		viewerColumn.setEditingSupport(new RightBindingColumnEditingSupport(treeViewerWithColumns));
		

		treeViewerWithColumns
				.setContentProvider(new FragmentBindingContentProvider(editor.getAdaptedFactory()));
			
		// resize all the columns
		//treeViewerWithColumns.expandAll();
		//for(TreeColumn c : columns) c.pack();
		//treeViewerWithColumns.collapseAll();
		
	}
	
	@Override
	protected List<ColumnDescriptor> getColumnDescriptors() {
		ArrayList<ColumnDescriptor> list = new ArrayList<ColumnDescriptor>();
		//list.add(new ColumnDescriptor("Name", null, "name"));
		return list;
	}
	
	
	class FragmentBindingContentProvider extends AdapterFactoryContentProvider {

		public FragmentBindingContentProvider(AdapterFactory adapterFactory) {
			super(adapterFactory);
		}

		@Override
		public Object[] getChildren(Object object) {
			if (object instanceof FragmentSubstitution) {
				return ((FragmentSubstitution)object).getBoundaryElementBinding().toArray();
			}
			return new Object[0];
		}

		@Override
		public boolean hasChildren(Object object) {
			if (object instanceof FragmentSubstitution) {
				return ((FragmentSubstitution)object).getBoundaryElementBinding().size() != 0;
			}
			return false;
		}

		@Override
		public Object[] getElements(Object object) {
			return getChildren(object);
		}
	}
	
	public class LeftBindingColumnLabelProvider extends ColumnLabelProvider {

		protected AdapterFactoryLabelProvider labelProvider;
		FontRegistry registry = new FontRegistry();

		private org.eclipse.swt.graphics.Color fNoEditColor;
		private final static String NOEDIT_STRING = "";

		public LeftBindingColumnLabelProvider() {
			super();
			this.fNoEditColor = new org.eclipse.swt.graphics.Color(Display.getCurrent(), 230, 230, 230);
			this.labelProvider = new AdapterFactoryLabelProvider(editor.getAdapterFactory());
		}

		@Override
		public String getText(Object element) {
			String result = NOEDIT_STRING;
			if (element instanceof ToBinding) {
				ToBinding tb = (ToBinding)element;
				result = "[" + labelProvider.getText(tb.getToPlacement().getOutsideBoundaryElement()) + "]";
				result += "." + tb.getToPlacement().getPropertyName();
			}
			else if (element instanceof FromBinding){
				FromBinding fb = (FromBinding)element;
				result = "[" + labelProvider.getText(fb.getFromReplacement().getInsideBoundaryElement()) + "]";
				result += "." + fb.getFromReplacement().getPropertyName();
			}
			return result;
		}

		@Override
		public org.eclipse.swt.graphics.Image getImage(Object element) {
			if (element instanceof ToBinding) {
				ToBinding tb = (ToBinding)element;
				return labelProvider.getImage(tb.getToPlacement());
			}
			else if (element instanceof FromBinding){
				FromBinding fb = (FromBinding)element;
				return labelProvider.getImage(fb.getFromReplacement());
			}
			return super.getImage(element);
		}

		@Override
		public void dispose() {
			this.fNoEditColor.dispose();
			super.dispose();
		}

		@Override
		public Font getFont(Object element) {
			if (element instanceof ToBinding) {
				return registry.getBold(Display.getCurrent().getSystemFont().getFontData()[0].getName());
			}
			else if (element instanceof FromBinding){
				return registry.getBold(Display.getCurrent().getSystemFont().getFontData()[0].getName());
			}
			return super.getFont(element);
		}

		@Override
		public Color getForeground(Object element) {
			if (element instanceof ToBinding) {
				return CVLUtil.getBindingForegroundColor((BoundaryElementBinding) element, CVLLabelProvider.ORANGE);
				//return CVLLabelProvider.ORANGE;
			}
			else if (element instanceof FromBinding){
				return CVLUtil.getBindingForegroundColor((BoundaryElementBinding) element, CVLLabelProvider.BLUE);
				//return CVLLabelProvider.BLUE;
			}
			return super.getForeground(element);
		}
		
		@Override
		public Color getBackground(Object element) {
			if (element instanceof ToBinding) {
				return super.getBackground(element);
			}
			else if (element instanceof FromBinding){
				return super.getBackground(element);
			}
			return fNoEditColor;
		}

	}
	
	public class MiddleBindingColumnLabelProvider extends ColumnLabelProvider {

		protected AdapterFactoryLabelProvider labelProvider;
		FontRegistry registry = new FontRegistry();

		private org.eclipse.swt.graphics.Color fNoEditColor;
		private final static String NOEDIT_STRING = "";

		public MiddleBindingColumnLabelProvider() {
			super();
			this.fNoEditColor = new org.eclipse.swt.graphics.Color(Display.getCurrent(), 230, 230, 230);
			this.labelProvider = new AdapterFactoryLabelProvider(editor.getAdapterFactory());
		}
		
		public EStructuralFeature getEStructuralFeature(EObject o, String property_name) {
			if(o == null) return null;
			EStructuralFeature property = null;
			for (EStructuralFeature sf : o.eClass().getEAllStructuralFeatures()) {
				if (sf.getName().equals(property_name) ) {
					property = sf;
					break;
				}
			}
			return property;
		}
		

		@Override
		public String getText(Object element) {
			if (element instanceof ToBinding) {
				ToBinding tb = (ToBinding)element;
				EStructuralFeature property = getEStructuralFeature(tb.getToPlacement().getOutsideBoundaryElement(), tb.getToPlacement().getPropertyName());
				if (property != null) {
					if (property.getUpperBound() != 1) return "+";
					else return "=";
				}
			}
			else if (element instanceof FromBinding){
				FromBinding fb = (FromBinding)element;
				EStructuralFeature property = getEStructuralFeature(fb.getFromReplacement().getInsideBoundaryElement(), fb.getFromReplacement().getPropertyName());
				if (property != null) {
					if (property.getUpperBound() != 1) return "+";
					else return "=";
				}
			}
			return "";
		}

		@Override
		public void dispose() {
			this.fNoEditColor.dispose();
			super.dispose();
		}

		@Override
		public Font getFont(Object element) {
			return registry.getBold(Display.getCurrent().getSystemFont().getFontData()[0].getName());
		}
		
		@Override
		public Color getBackground(Object element) {
			if (element instanceof ToBinding) {
				return super.getBackground(element);
			}
			else if (element instanceof FromBinding){
				return super.getBackground(element);
			}
			return fNoEditColor;
		}

	}
	
	
	
	public class RightBindingColumnLabelProvider extends ColumnLabelProvider {

		protected AdapterFactoryLabelProvider labelProvider;
		FontRegistry registry = new FontRegistry();

		private org.eclipse.swt.graphics.Color fNoEditColor;
		private final static String NOEDIT_STRING = "";
		
		public RightBindingColumnLabelProvider() {
			super();
			this.fNoEditColor = new org.eclipse.swt.graphics.Color(Display.getCurrent(), 230, 230, 230);
			this.labelProvider = new AdapterFactoryLabelProvider(editor.getAdapterFactory());
		}

		public String labelForToReplacement(ToReplacement tr) {
			if(tr == null) return "<< NONE >>";
			String result = "{";
			for(EObject o : tr.getInsideBoundaryElement()) {
				result += labelProvider.getText(o) + ", ";
			}
			if (result.length() > 2) result = result.substring(0, result.length()-2);
		
			result += "}";
			return result;
		}
		
		public String labelForFromPlacement(FromPlacement fp) {
			if(fp == null) return "<< NONE >>";
			String result = "{";
			for(EObject o : fp.getOutsideBoundaryElement()) {
				result += labelProvider.getText(o) + ", ";
			}
			if (result.length() > 2) result = result.substring(0, result.length()-2);
		
			result += "}";
			return result;
		}
		
		@Override
		public String getText(Object element) {
			String result = NOEDIT_STRING;
			if (element instanceof ToBinding) {
				ToBinding tb = (ToBinding)element;
				result = labelForToReplacement(tb.getToReplacement());
			}
			else if (element instanceof FromBinding){
				FromBinding fb = (FromBinding)element;
				result = labelForFromPlacement(fb.getFromPlacement());
			}
			return result;
		}

		@Override
		public org.eclipse.swt.graphics.Image getImage(Object element) {
			if (element instanceof ToBinding) {
				ToBinding tb = (ToBinding)element;
				if(tb.getToReplacement() == null) return null;
				return labelProvider.getImage(tb.getToReplacement());
			}
			else if (element instanceof FromBinding){
				FromBinding fb = (FromBinding)element;
				if(fb.getFromPlacement() == null) return null;
				return labelProvider.getImage(fb.getFromPlacement());
			}
			return super.getImage(element);
		}

		@Override
		public void dispose() {
			this.fNoEditColor.dispose();
			super.dispose();
		}

		@Override
		public Font getFont(Object element) {
			if (element instanceof ToBinding) {
				return registry.getBold(Display.getCurrent().getSystemFont().getFontData()[0].getName());
			}
			else if (element instanceof FromBinding){
				return registry.getBold(Display.getCurrent().getSystemFont().getFontData()[0].getName());
			}
			return super.getFont(element);
		}

	
		@Override
		public Color getForeground(Object element) {
			if (element instanceof ToBinding) {
				return CVLUtil.getBindingForegroundColor((BoundaryElementBinding) element, CVLLabelProvider.BLUE);
				//return CVLLabelProvider.BLUE;
			}
			else if (element instanceof FromBinding){
				return CVLUtil.getBindingForegroundColor((BoundaryElementBinding) element, CVLLabelProvider.ORANGE);
				//return CVLLabelProvider.ORANGE;
			}
			return super.getForeground(element);
		}

		@Override
		public Color getBackground(Object element) {
			if (element instanceof ToBinding) {
				return super.getBackground(element);
			}
			else if (element instanceof FromBinding){
				return super.getBackground(element);
			}
			return fNoEditColor;
		}
	}
	
	
	public class RightBindingColumnEditingSupport extends EditingSupport {

		protected RightBindingColumnLabelProvider lp = new RightBindingColumnLabelProvider();
		protected Hashtable<ToBinding, ArrayList<ToReplacement>> toChoices = new Hashtable<ToBinding, ArrayList<ToReplacement>>();
		protected Hashtable<FromBinding, ArrayList<FromPlacement>> fromChoices = new Hashtable<FromBinding, ArrayList<FromPlacement>>();
		
		protected String[] getToChoices(ToBinding tb) {
			
			ToPlacement tp = tb.getToPlacement();
			if (tp == null) return new String[0];
			
			if (tb.eContainer() != null && tb.eContainer() instanceof FragmentSubstitution) {
				FragmentSubstitution substitution = (FragmentSubstitution)tb.eContainer();
				ArrayList<ToReplacement> list = CVLUtil.getCompatibleToReplacement(CVLUtil.getConcreteReplacement(substitution.getReplacement()), tp);
				toChoices.put(tb, list);
				String[] result = new String[list.size()+1];
				result[0]= "<< NONE >>";
				for(int i=0; i<list.size(); i++) {
					result[i+1] = lp.labelForToReplacement(list.get(i));
					if (result[i+1].length() > 30) {
						result[i+1] = result[i+1].substring(0, 28) + "...";
					}
				}
				return result;
			}
			else return new String[0];
			
		}
		
		protected String[] getFromChoices(FromBinding fb) {
			
			FromReplacement fr = fb.getFromReplacement();
			if (fr == null) return new String[0];
			
			if (fb.eContainer() != null && fb.eContainer() instanceof FragmentSubstitution) {
				FragmentSubstitution substitution = (FragmentSubstitution)fb.eContainer();
					if (substitution == null) return new String[0];
					if(substitution.getPlacement() == null) return new String[0];
					ArrayList<FromPlacement> list = CVLUtil.getCompatibleFromPlacement(substitution.getPlacement(), fr);
					fromChoices.put(fb, list);
					String[] result = new String[list.size()+1];
					result[0]= "<< NONE >>";
					for(int i=0; i<list.size(); i++) {
						result[i+1] = lp.labelForFromPlacement(list.get(i));
						if (result[i+1].length() > 30) {
							result[i+1] = result[i+1].substring(0, 28) + "...";
						}
					}
					return result;
			}
			return new String[0];
			
		}
		
		public RightBindingColumnEditingSupport(ColumnViewer viewer) {
			super(viewer);
		}

		@Override
		protected boolean canEdit(Object element) {
			if (element instanceof ToBinding) return true;
			else if (element instanceof FromBinding) return true;
			return false;
		}

		@Override
		protected CellEditor getCellEditor(Object element) {
			CellEditor result = null;
			if (element instanceof ToBinding) {
				result = new ComboBoxCellEditor((Composite) getViewer().getControl(), getToChoices((ToBinding)element));
			}
			else if (element instanceof FromBinding) {
				result = new ComboBoxCellEditor((Composite) getViewer().getControl(), getFromChoices((FromBinding)element));
			}
			return result;
		}

		@Override
		protected Object getValue(Object element) {
			Object result = null;
			if (element instanceof ToBinding) {
				ToBinding tb = (ToBinding)element;
				if(tb == null || tb.getToPlacement() == null) return 0;
				if(tb.getToReplacement() == null) return 0;
				return toChoices.get(tb).indexOf(tb.getToReplacement()) + 1;
			}
			else if (element instanceof FromBinding) {
				FromBinding fb = (FromBinding)element;
				if(fb == null || fb.getFromReplacement() == null) return 0;
				if(fb.getFromPlacement() == null) return 0;
				return fromChoices.get(fb).indexOf(fb.getFromPlacement()) + 1;
			}
			return result;
		}

		@Override
		protected void setValue(Object element, Object value) {
			if (value == null) return; // Need this check because somehow this gets called with value null
				
			if (element instanceof ToBinding) {
				ArrayList<ToReplacement> choices = toChoices.get(element);
				if(choices == null) return;
				int v = ((Integer)value).intValue();
				if(v >= choices.size() + 1) return;
				ToReplacement tr = null;
				if(v > 0) tr = choices.get(v-1);
				SetCommand cmd = new SetCommand(editor.getEditingDomain(), (ToBinding)element, CvlPackage.eINSTANCE.getToBinding_ToReplacement(), tr);
				editor.getEditingDomain().getCommandStack().execute(cmd);
				getViewer().update(element, null);
			}
			else if (element instanceof FromBinding) {
				ArrayList<FromPlacement> choices = fromChoices.get(element);
				if(choices == null) return;
				int v = ((Integer)value).intValue();
				if(v >= choices.size() + 1) return;
				FromPlacement fp = null;
				if(v > 0) fp = choices.get(v-1);
				SetCommand cmd = new SetCommand(editor.getEditingDomain(), (FromBinding)element, CvlPackage.eINSTANCE.getFromBinding_FromPlacement(), fp);
				editor.getEditingDomain().getCommandStack().execute(cmd);
				getViewer().update(element, null);
			}
		}
	}
	
}
