package cvl.runtime;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.mofscript.MOFScriptModel.MOFScriptSpecification;
import org.eclipse.mofscript.parser.MofScriptParseError;
import org.eclipse.mofscript.parser.ParserUtil;
import org.eclipse.mofscript.runtime.ExecutionManager;
import org.eclipse.mofscript.runtime.ExecutionMessageListener;
import org.eclipse.mofscript.runtime.MofScriptExecutionException;

import cvl.CVLModel;

public class GenerateRepresentation implements ExecutionMessageListener  {

	ParserUtil parserUtil = new ParserUtil();
	ExecutionManager execMgr = ExecutionManager.getExecutionManager();
	XMIResourceFactoryImpl _xmiFac = new XMIResourceFactoryImpl(); 
	EObject sourceModel = null;	
	public String baseExt = "";   //used for file extension when saving output model
	public List<String> errorMsg = new ArrayList<String>();  //used to store error messages

	public GenerateRepresentation(String fileName, String transformationPath) {
		baseExt = "";
		errorMsg.clear();		
		loadTransformation(transformationPath);
		loadSourceModel(fileName);	
	}

	void loadTransformation(String transformationPath){
		File file = new File (transformationPath);
		MOFScriptSpecification spec = parserUtil.parse(file, true);
		int errorCount = ParserUtil.getModelChecker().getErrorCount();
		Iterator errorIt = ParserUtil.getModelChecker().getErrors(); // Iterator of MofScriptParseError objects

		System.out.println ("Parsing result: " + errorCount + " errors"); 
		if (errorCount > 0) {

			for (;errorIt.hasNext();) {
				MofScriptParseError parseError = (MofScriptParseError) errorIt.next();
				System.out.println("\t \t: Error: " + parseError.toString());
			} 
			return; 
		}
	}

	void loadSourceModel(String fileName){
		
		File sourceModelFile = new File (fileName);
		execMgr.loadSourceModel(sourceModelFile);
		
		ResourceSet rSet = new ResourceSetImpl ();
		rSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put("*", _xmiFac);
		URI uri = URI.createFileURI(sourceModelFile.getAbsolutePath());
		Resource resource = rSet.getResource(uri, true);

		if (resource != null) {
			if (resource.getContents().size() > 0) {
				sourceModel = (EObject) resource.getContents().get(0);
				
				//get the file extension of the base model
				CVLModel c = (CVLModel) sourceModel;
				
				if (c.getBase() != null){
					baseExt = c.getBase().eResource().getURI().fileExtension();
				}
			}
		}
	
	}
	
	void execute(String path){
	//	execMgr.addSourceModel(sourceModel);
		
		EcoreUtil.resolveAll(sourceModel);
		execMgr.setRootDirectory(path);
		execMgr.setUseFileModel(false);
		execMgr.getExecutionStack().addOutputMessageListener(this);   
		try {
			execMgr.executeTransformation();			
		} catch (MofScriptExecutionException mex) {
			mex.printStackTrace();
		}
	}
	
    /**
     * ExecutionMessageListener interface operations
     */
    public void executionMessage (String type, String description) {
    	String errorStart = "## Error: ";
    
    	//error handling
    	if (description.startsWith(errorStart)){
    		if (errorMsg.size()>0){
    			String s = errorMsg.get(errorMsg.size()-1);
    			if (!description.equals(s)){   //workaround for skipping several of the same message
    				errorMsg.add(description.substring(errorStart.length()));
    			}
    		}else{
    			errorMsg.add(description.substring(errorStart.length()));
    		}
    	}
    }

}
