package cvl.util;

import java.util.ArrayList;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.swt.graphics.Color;

import cvl.BoundaryElementBinding;
import cvl.FragmentSubstitution;
import cvl.FromBinding;
import cvl.FromPlacement;
import cvl.FromReplacement;
import cvl.PlacementBoundaryElement;
import cvl.PlacementFragment;
import cvl.ReplacementBoundaryElement;
import cvl.ReplacementFragment;
import cvl.ReplacementFragmentAbstract;
import cvl.ReplacementFragmentReference;
import cvl.ToBinding;
import cvl.ToPlacement;
import cvl.ToReplacement;
import cvl.editor.CVLLabelProvider;
import cvl.impl.CvlFactoryImpl;

public class CVLUtil {

	
	//method to recursively include contained objects into a fragment
	//Franck, please don't laugh when you see this code ;)
	public static ArrayList<EObject> includeContainment(ArrayList<EObject> objects){
		ArrayList<EObject> processing = new ArrayList<EObject>();
		ArrayList<EObject> result = new ArrayList<EObject>();
		
		processing.addAll(objects);
		
		while (!processing.isEmpty()){
			EObject obj = processing.get(0);
			processing.remove(0);
	
			result.add(obj);
			
			EList<EReference> refs = obj.eClass().getEAllContainments();
			
			for (EReference ref : refs) {
				if (ref.getUpperBound() == 1){
					Object target = obj.eGet(ref);
					if (target instanceof EObject && !objects.contains(target)){
						result.add((EObject)target);
						processing.add((EObject)target);
					}
				}else{
					EList<?> list = (EList<?>)obj.eGet(ref);
					
					for (Object target : list) {
						if (target instanceof EObject && !objects.contains(target)){
							result.add((EObject)target);
							processing.add((EObject)target);
						}
					}
				}
			}
		}
		
		return removeDuplicates(result);
	}
	
	//quick fix to solve duplicate problems with the papyrus editor (multidiagram editors)
	public static ArrayList<EObject> removeDuplicates(ArrayList<EObject> objects){
		ArrayList<EObject> result = new ArrayList<EObject>();
		
		for (EObject eObject : objects) {
			if (!result.contains(eObject)){
				result.add(eObject);
			}
		}
		return result;
	}
	
	//return green if the there is only one choice for the binding, else return default (Color c).
	//Please don't laugh here either ;)
	public static Color getBindingForegroundColor(BoundaryElementBinding b, Color c){
		if (b instanceof ToBinding){
			ToBinding tb = (ToBinding) b;
			ToPlacement tp = tb.getToPlacement();
			if (tp == null) {
				return c;
			}
			if (tb.eContainer() != null && tb.eContainer() instanceof FragmentSubstitution) {
				FragmentSubstitution substitution = (FragmentSubstitution)tb.eContainer();
				ArrayList<ToReplacement> list = CVLUtil.getCompatibleToReplacement(CVLUtil.getConcreteReplacement(substitution.getReplacement()), tp);
				if (list.size() > 1){
					return c;
				}
				return CVLLabelProvider.GREEN;
			}
		}else if (b instanceof FromBinding){
			FromBinding fb = (FromBinding) b;
			FromReplacement fr = fb.getFromReplacement();
			if (fr == null) {
				return c;
			}
			
			if (fb.eContainer() != null && fb.eContainer() instanceof FragmentSubstitution) {
				FragmentSubstitution substitution = (FragmentSubstitution)fb.eContainer();
					if(substitution.getPlacement() == null) {
						return c;
					}
					ArrayList<FromPlacement> list = CVLUtil.getCompatibleFromPlacement(substitution.getPlacement(), fr);
					if (list.size() > 1){
						return c;
					}
					return CVLLabelProvider.GREEN;
			}
		}
		return c;
	}
	
	public static PlacementFragment createPlacementFragment(ArrayList<EObject> inside) {
		PlacementFragment result = CvlFactoryImpl.init().createPlacementFragment();
		ArrayList<EObject> tgts = new ArrayList<EObject>();
		ArrayList<Resource> processed = new ArrayList<Resource>();
		for(EObject o : inside) {
			for (EReference p : o.eClass().getEAllReferences()) {
				if (p.getUpperBound() == 1) {
					Object target = o.eGet(p);
					if (target instanceof EObject && !inside.contains((EObject)target)) {
						// check if a similar FromPlacement already exists
						Boolean found = false;
						for (PlacementBoundaryElement pbe : result.getBoundaryElement()) {
							if (pbe instanceof FromPlacement) {
								FromPlacement fp = (FromPlacement)pbe;
								if(fp.getOutsideBoundaryElement().size() == 1 && 
										fp.getOutsideBoundaryElement().contains((EObject)target)) {
									found = true;
									break;
								}
							}
						}
						// if not create a new one
						if(!found) {
							FromPlacement fp = CvlFactoryImpl.init().createFromPlacement();
							fp.getOutsideBoundaryElement().add((EObject)target);
							fp.setName(":" + ((EObject)target).eClass().getName() + "[" + fp.getOutsideBoundaryElement().size() + "]");
							result.getBoundaryElement().add(fp);
						}
					}
				}
				else {
					EList<?> l = (EList<?>)o.eGet(p);
					tgts.clear();
					for(Object target : l) {
						if (target instanceof EObject && !inside.contains((EObject)target)) {
							tgts.add((EObject)target);
						}
					}
					if(!tgts.isEmpty()) {
						// check if a similar FromPlacement already exists
						Boolean found = false;
						for (PlacementBoundaryElement pbe : result.getBoundaryElement()) {
							if (pbe instanceof FromPlacement) {
								FromPlacement fp = (FromPlacement)pbe;
								if(fp.getOutsideBoundaryElement().size() == tgts.size() && 
										fp.getOutsideBoundaryElement().containsAll(tgts)) {
									found = true;
									break;
								}
							}
						}
						// if not create a new one
						if(!found) {
							FromPlacement fp = CvlFactoryImpl.init().createFromPlacement();
							for(EObject t : tgts) fp.getOutsideBoundaryElement().add(t);
							fp.setName(":" + p.getEType().getName() + "[" + fp.getOutsideBoundaryElement().size() + "]");
							result.getBoundaryElement().add(fp);
						}
					}
				}
			}
			
			if (!processed.contains(o.eResource())) {
				processed.add(o.eResource());
				TreeIterator<EObject> it = o.eResource().getAllContents();
				while(it.hasNext()) {
					EObject src = it.next();
					if (inside.contains(src)) continue;
					for (EReference p : src.eClass().getEAllReferences()) {
						if (p.getUpperBound() == 1) {
							Object target = src.eGet(p);
							if (target instanceof EObject && inside.contains((EObject)target)) {
								ToPlacement tp = CvlFactoryImpl.init().createToPlacement();
								tp.setOutsideBoundaryElement(src);
								tp.getInsideBoundaryElement().add((EObject)target);
								tp.setPropertyName(p.getName());
								tp.setName("[:" + tp.getOutsideBoundaryElement().eClass().getName() + "]." + tp.getPropertyName());
								result.getBoundaryElement().add(tp);
							}
						}
						else {
							EList<?> l = (EList<?>)src.eGet(p);
							tgts.clear();
							for(Object target : l) {
								if (target instanceof EObject && inside.contains((EObject)target)) {
									tgts.add((EObject)target);
								}
							}
							if(!tgts.isEmpty()) {
								ToPlacement tp = CvlFactoryImpl.init().createToPlacement();
								tp.setOutsideBoundaryElement(src);
								for(EObject t : tgts) tp.getInsideBoundaryElement().add(t);
								tp.setPropertyName(p.getName());
								tp.setName("[:" + tp.getOutsideBoundaryElement().eClass().getName() + "]." + tp.getPropertyName());
								result.getBoundaryElement().add(tp);
							}
						}
					}
				}
			}
		}
		return result;
	}
	
	public static ReplacementFragment createReplacementFragment(ArrayList<EObject> inside) {
		ReplacementFragment result = CvlFactoryImpl.init().createReplacementFragment();
		ArrayList<EObject> tgts = new ArrayList<EObject>();
		ArrayList<Resource> processed = new ArrayList<Resource>();
		for(EObject o : inside) {
			for (EReference p : o.eClass().getEAllReferences()) {
				if (p.getUpperBound() == 1) {
					Object target = o.eGet(p);
					if (target instanceof EObject && !inside.contains((EObject)target)) {
						FromReplacement fr = CvlFactoryImpl.init().createFromReplacement();
						fr.setPropertyName(p.getName());
						fr.getOutsideBoundaryElement().add((EObject)target);
						fr.setInsideBoundaryElement(o);
						fr.setName("[:" + fr.getInsideBoundaryElement().eClass().getName() + "]." + fr.getPropertyName());
						result.getBoundaryElement().add(fr);
					}
				}
				else {
					EList<?> l = (EList<?>)o.eGet(p);
					tgts.clear();
					for(Object target : l) {
						if (target instanceof EObject && !inside.contains((EObject)target)) {
							tgts.add((EObject)target);
						}
					}
					if(!tgts.isEmpty()) {
						FromReplacement fr = CvlFactoryImpl.init().createFromReplacement();
						fr.setPropertyName(p.getName());
						for(EObject t : tgts) fr.getOutsideBoundaryElement().add(t);
						fr.setInsideBoundaryElement(o);
						fr.setName("[:" + fr.getInsideBoundaryElement().eClass().getName() + "]." + fr.getPropertyName());
						result.getBoundaryElement().add(fr);
					}
				}
			}
			
			if (!processed.contains(o.eResource())) {
				processed.add(o.eResource());
				TreeIterator<EObject> it = o.eResource().getAllContents();
				while(it.hasNext()) {
					EObject src = it.next();
					if (inside.contains(src)) continue;
					for (EReference p : src.eClass().getEAllReferences()) {
						if (p.getUpperBound() == 1) {
							Object target = src.eGet(p);
							if (target instanceof EObject && inside.contains((EObject)target)) {
								// check if a similar ToReplacement already exists
								Boolean found = false;
								for (ReplacementBoundaryElement rbe : result.getBoundaryElement()) {
									if (rbe instanceof ToReplacement) {
										ToReplacement tr = (ToReplacement)rbe;
										if(tr.getInsideBoundaryElement().size() == 1 && 
												tr.getInsideBoundaryElement().contains((EObject)target)) {
											found = true;
											break;
										}
									}
								}
								// if not create a new one
								if(!found) {
									ToReplacement tr = CvlFactoryImpl.init().createToReplacement();
									tr.getInsideBoundaryElement().add((EObject)target);
									tr.setName(":" + ((EObject)target).eClass().getName() + "[" + tr.getInsideBoundaryElement().size() + "]");
									result.getBoundaryElement().add(tr);
								}
							}
						}
						else {
							EList<?> l = (EList<?>)src.eGet(p);
							tgts.clear();
							for(Object target : l) {
								if (target instanceof EObject && inside.contains((EObject)target)) {
									tgts.add((EObject)target);
								}
							}
							if(!tgts.isEmpty()) {
								// check if a similar ToReplacement already exists
								Boolean found = false;
								for (ReplacementBoundaryElement rbe : result.getBoundaryElement()) {
									if (rbe instanceof ToReplacement) {
										ToReplacement tr = (ToReplacement)rbe;
										if(tr.getInsideBoundaryElement().size() == tgts.size() && 
												tr.getInsideBoundaryElement().containsAll(tgts)) {
											found = true;
											break;
										}
									}
								}
								// if not create a new one
								if(!found) {
									ToReplacement tr = CvlFactoryImpl.init().createToReplacement();
									for(EObject t : tgts) tr.getInsideBoundaryElement().add(t);
									tr.setName(":" + p.getEType().getName() + "[" + tr.getInsideBoundaryElement().size() + "]");
									result.getBoundaryElement().add(tr);
								}
							}
						}
					}
				}
			}
		}
		return result;
	}
	
	public static ArrayList<EObject> getPlacementFragmentContent(PlacementFragment pf) {
		ArrayList<EObject> result = new ArrayList<EObject>();
		ArrayList<EObject> out = new ArrayList<EObject>();
		ArrayList<EObject> toProcess = new ArrayList<EObject>();
		ArrayList<ToPlacement> tps = new ArrayList<ToPlacement>();
		ArrayList<FromPlacement> fps = new ArrayList<FromPlacement>();
		for (PlacementBoundaryElement be : pf.getBoundaryElement()) {
			if (be instanceof ToPlacement) tps.add((ToPlacement)be);
			else fps.add((FromPlacement)be);
		}
		for(FromPlacement fp : fps) out.addAll(fp.getOutsideBoundaryElement());
		for(ToPlacement tp : tps) toProcess.addAll(tp.getInsideBoundaryElement());
		
		while(!toProcess.isEmpty()) {
			EObject current = toProcess.get(0);
			toProcess.remove(0);
			result.add(current);
			for(EReference p : current.eClass().getEAllReferences()) {
				if(p.getUpperBound() == 1) {
					Object target = current.eGet(p);
					if (target instanceof EObject && !result.contains(target) && 
							!out.contains(target) && !toProcess.contains(target)) {
						toProcess.add((EObject)target);
					}
				}
				else {
					EList<?> l = (EList<?>)current.eGet(p);
					for(Object target : l) {
						if (target instanceof EObject && !result.contains(target) && 
								!out.contains(target) && !toProcess.contains(target)) {
							toProcess.add((EObject)target);
						}
					}
				}
			}	
		}
		return result;
	}
	
	public static ArrayList<EObject> getReplacementFragmentContent(ReplacementFragment rf) {
		ArrayList<EObject> result = new ArrayList<EObject>();
		ArrayList<EObject> out = new ArrayList<EObject>();
		ArrayList<EObject> toProcess = new ArrayList<EObject>();
		ArrayList<ToReplacement> trs = new ArrayList<ToReplacement>();
		ArrayList<FromReplacement> frs = new ArrayList<FromReplacement>();
		for (ReplacementBoundaryElement be : rf.getBoundaryElement()) {
			if (be instanceof ToReplacement) trs.add((ToReplacement)be);
			else frs.add((FromReplacement)be);
		}
		for(FromReplacement fr : frs) out.addAll(fr.getOutsideBoundaryElement());
		for(ToReplacement tr : trs) toProcess.addAll(tr.getInsideBoundaryElement());
		
		while(!toProcess.isEmpty()) {
			EObject current = toProcess.get(0);
			toProcess.remove(0);
			result.add(current);
			for(EReference p : current.eClass().getEAllReferences()) {
				if(p.getUpperBound() == 1) {
					Object target = current.eGet(p);
					if (target instanceof EObject && !result.contains(target) && 
							!out.contains(target) && !toProcess.contains(target)) {
						toProcess.add((EObject)target);
					}
				}
				else {
					EList<?> l = (EList<?>)current.eGet(p);
					for(Object target : l) {
						if (target instanceof EObject && !result.contains(target) && 
								!out.contains(target) && !toProcess.contains(target)) {
							toProcess.add((EObject)target);
						}
					}
				}
			}	
		}
		return result;
	}
	
	public static ArrayList<ToReplacement> getCompatibleToReplacement(ReplacementFragment replacement, ToPlacement toPlacement) {
		ArrayList<ToReplacement> result = new ArrayList<ToReplacement>();
		// Check that the parameters. If anything is incomplete return an empty list
		if(replacement == null) return result;
		if (toPlacement.getOutsideBoundaryElement() == null || toPlacement.getPropertyName() == null) return result;
		// Search for the property
		EStructuralFeature property = null;
		for (EStructuralFeature sf : toPlacement.getOutsideBoundaryElement().eClass().getEAllStructuralFeatures()) {
			if (sf.getName().equals(toPlacement.getPropertyName()) ) {
				property = sf;
				break;
			}
		}
		// return an empty list if not found
		if (property == null) return result;
		// Select the compatible ToReplacement
		for(ReplacementBoundaryElement be : replacement.getBoundaryElement()) {
			if(be instanceof ToReplacement) {
				ToReplacement tr = (ToReplacement)be;
				Boolean isCompatible = true;
				// check multiplicity
				if(property.getUpperBound() == 1 && tr.getInsideBoundaryElement().size()>1) continue;
				// check types of objects
				for(EObject o : tr.getInsideBoundaryElement()) {
					if(!(isCompatible = property.getEType().isInstance(o))) break;
				}
				if(isCompatible) result.add(tr);
			}
		}
		return result;
	}
	
	public static ArrayList<FromPlacement> getCompatibleFromPlacement(PlacementFragment placement, FromReplacement fromReplacement) {
		ArrayList<FromPlacement> result = new ArrayList<FromPlacement>();
		// Check that the parameters. If anything is incomplete return an empty list
		if(placement == null) return result;
		if (fromReplacement.getInsideBoundaryElement() == null || fromReplacement.getPropertyName() == null) return result;
		// Search for the property
		EStructuralFeature property = null;
		for (EStructuralFeature sf : fromReplacement.getInsideBoundaryElement().eClass().getEAllStructuralFeatures()) {
			if (sf.getName().equals(fromReplacement.getPropertyName()) ) {
				property = sf;
				break;
			}
		}
		// return an empty list if not found
		if (property == null) return result;
		// Select the compatible ToReplacement
		for(PlacementBoundaryElement be : placement.getBoundaryElement()) {
			if(be instanceof FromPlacement) {
				FromPlacement fp = (FromPlacement)be;
				Boolean isCompatible = true;
				// check multiplicity
				if(property.getUpperBound() == 1 && fp.getOutsideBoundaryElement().size()>1) continue;
				// check types of objects
				for(EObject o : fp.getOutsideBoundaryElement()) {
					if(!(isCompatible = property.getEType().isInstance(o))) break;
				}
				if(isCompatible) result.add(fp);
			}
		}
		return result;
	}
	
	public static ReplacementFragment getConcreteReplacement(ReplacementFragmentAbstract abs) {
		if (abs instanceof ReplacementFragment) return (ReplacementFragment)abs;
		ReplacementFragmentReference ref = (ReplacementFragmentReference)abs;
		return ref.getRfRef();
	}
	
	public static ArrayList<BoundaryElementBinding> createDefaultBindings(PlacementFragment placement, ReplacementFragment replacement) {
		ArrayList<BoundaryElementBinding> result = new ArrayList<BoundaryElementBinding>();
		
		if (placement == null || replacement == null) return result;
		
		for (PlacementBoundaryElement be : placement.getBoundaryElement()) {
			if (be instanceof ToPlacement) {
				ToPlacement tp = (ToPlacement)be;
				ToBinding tb = CvlFactoryImpl.init().createToBinding();
				tb.setToPlacement(tp);
				ArrayList<ToReplacement> trs = getCompatibleToReplacement(replacement, tp);
				if(trs.size() > 0) tb.setToReplacement(trs.get(0));
				tb.setName("[:" + tp.getOutsideBoundaryElement().eClass().getName() + "]." + tp.getPropertyName());
				result.add(tb);
			}
		}
		for (ReplacementBoundaryElement be : replacement.getBoundaryElement()) {
			if (be instanceof FromReplacement) {
				FromReplacement fr = (FromReplacement)be;
				FromBinding fb = CvlFactoryImpl.init().createFromBinding();
				fb.setFromReplacement(fr);
				ArrayList<FromPlacement> fps = getCompatibleFromPlacement(placement, fr);
				if(fps.size() > 0) fb.setFromPlacement(fps.get(0));
				fb.setName("[:" + fr.getInsideBoundaryElement().eClass().getName() + "]." + fr.getPropertyName());
				result.add(fb);
			}
		}
		return result;
	}
	
	
}
